# -*- coding: utf-8 -*-
#*********************************************************************
#*
#* $Id: yocto_serialport.py 59222 2024-02-05 15:50:11Z seb $
#*
#* Implements yFindSerialPort(), the high-level API for SerialPort functions
#*
#* - - - - - - - - - License information: - - - - - - - - -
#*
#*  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
#*
#*  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
#*  non-exclusive license to use, modify, copy and integrate this
#*  file into your software for the sole purpose of interfacing
#*  with Yoctopuce products.
#*
#*  You may reproduce and distribute copies of this file in
#*  source or object form, as long as the sole purpose of this
#*  code is to interface with Yoctopuce products. You must retain
#*  this notice in the distributed source file.
#*
#*  You should refer to Yoctopuce General Terms and Conditions
#*  for additional information regarding your rights and
#*  obligations.
#*
#*  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
#*  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
#*  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
#*  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
#*  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
#*  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
#*  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
#*  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
#*  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
#*  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
#*  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
#*  WARRANTY, OR OTHERWISE.
#*
#*********************************************************************/


__docformat__ = 'restructuredtext en'
from yoctopuce.yocto_api import *



#--- (generated code: YSnoopingRecord class start)
#noinspection PyProtectedMember
class YSnoopingRecord(object):
    #--- (end of generated code: YSnoopingRecord class start)
    #--- (generated code: YSnoopingRecord definitions)
    #--- (end of generated code: YSnoopingRecord definitions)

    def __init__(self, json_str):
        #--- (generated code: YSnoopingRecord attributes)
        self._tim = 0
        self._pos = 0
        self._dir = 0
        self._msg = ''
        #--- (end of generated code: YSnoopingRecord attributes)
        json = YJSONObject(json_str, 0, len(json_str))
        json.parse()
        if json_val.has("t"):
            self._tim = json.getInt("t")
        if json_val.has("p"):
            self._pos = json.getInt("p")
        if json_val.has("m"):
            m = json.getString("m")
            if m[0] == '<':
                self._dir = 1
            else:
                self._dir = 0
            self._msg = m[1:]

    #--- (generated code: YSnoopingRecord implementation)
    def get_time(self):
        """
        Returns the elapsed time, in ms, since the beginning of the preceding message.

        @return the elapsed time, in ms, since the beginning of the preceding message.
        """
        return self._tim

    def get_pos(self):
        """
        Returns the absolute position of the message end.

        @return the absolute position of the message end.
        """
        return self._pos

    def get_direction(self):
        """
        Returns the message direction (RX=0, TX=1).

        @return the message direction (RX=0, TX=1).
        """
        return self._dir

    def get_message(self):
        """
        Returns the message content.

        @return the message content.
        """
        return self._msg

#--- (end of generated code: YSnoopingRecord implementation)

#--- (generated code: YSnoopingRecord functions)
#--- (end of generated code: YSnoopingRecord functions)



#--- (generated code: YSerialPort class start)
#noinspection PyProtectedMember
class YSerialPort(YFunction):
    """
    The YSerialPort class allows you to fully drive a Yoctopuce serial port.
    It can be used to send and receive data, and to configure communication
    parameters (baud rate, bit count, parity, flow control and protocol).
    Note that Yoctopuce serial ports are not exposed as virtual COM ports.
    They are meant to be used in the same way as all Yoctopuce devices.

    """
    #--- (end of generated code: YSerialPort class start)
    #--- (generated code: YSerialPort return codes)
    #--- (end of generated code: YSerialPort return codes)
    #--- (generated code: YSerialPort dlldef)
    #--- (end of generated code: YSerialPort dlldef)
    #--- (generated code: YSerialPort definitions)
    RXCOUNT_INVALID = YAPI.INVALID_UINT
    TXCOUNT_INVALID = YAPI.INVALID_UINT
    ERRCOUNT_INVALID = YAPI.INVALID_UINT
    RXMSGCOUNT_INVALID = YAPI.INVALID_UINT
    TXMSGCOUNT_INVALID = YAPI.INVALID_UINT
    LASTMSG_INVALID = YAPI.INVALID_STRING
    CURRENTJOB_INVALID = YAPI.INVALID_STRING
    STARTUPJOB_INVALID = YAPI.INVALID_STRING
    JOBMAXTASK_INVALID = YAPI.INVALID_UINT
    JOBMAXSIZE_INVALID = YAPI.INVALID_UINT
    COMMAND_INVALID = YAPI.INVALID_STRING
    PROTOCOL_INVALID = YAPI.INVALID_STRING
    SERIALMODE_INVALID = YAPI.INVALID_STRING
    VOLTAGELEVEL_OFF = 0
    VOLTAGELEVEL_TTL3V = 1
    VOLTAGELEVEL_TTL3VR = 2
    VOLTAGELEVEL_TTL5V = 3
    VOLTAGELEVEL_TTL5VR = 4
    VOLTAGELEVEL_RS232 = 5
    VOLTAGELEVEL_RS485 = 6
    VOLTAGELEVEL_TTL1V8 = 7
    VOLTAGELEVEL_SDI12 = 8
    VOLTAGELEVEL_INVALID = -1
    #--- (end of generated code: YSerialPort definitions)

    def __init__(self, func):
        super(YSerialPort, self).__init__(func)
        self._className = 'SerialPort'
        #--- (generated code: YSerialPort attributes)
        self._callback = None
        self._rxCount = YSerialPort.RXCOUNT_INVALID
        self._txCount = YSerialPort.TXCOUNT_INVALID
        self._errCount = YSerialPort.ERRCOUNT_INVALID
        self._rxMsgCount = YSerialPort.RXMSGCOUNT_INVALID
        self._txMsgCount = YSerialPort.TXMSGCOUNT_INVALID
        self._lastMsg = YSerialPort.LASTMSG_INVALID
        self._currentJob = YSerialPort.CURRENTJOB_INVALID
        self._startupJob = YSerialPort.STARTUPJOB_INVALID
        self._jobMaxTask = YSerialPort.JOBMAXTASK_INVALID
        self._jobMaxSize = YSerialPort.JOBMAXSIZE_INVALID
        self._command = YSerialPort.COMMAND_INVALID
        self._protocol = YSerialPort.PROTOCOL_INVALID
        self._voltageLevel = YSerialPort.VOLTAGELEVEL_INVALID
        self._serialMode = YSerialPort.SERIALMODE_INVALID
        self._rxptr = 0
        self._rxbuff = bytearray()
        self._rxbuffptr = 0
        self._eventPos = 0
        self._eventCallback = None
        #--- (end of generated code: YSerialPort attributes)

    #--- (generated code: YSerialPort implementation)
    def _parseAttr(self, json_val):
        if json_val.has("rxCount"):
            self._rxCount = json_val.getInt("rxCount")
        if json_val.has("txCount"):
            self._txCount = json_val.getInt("txCount")
        if json_val.has("errCount"):
            self._errCount = json_val.getInt("errCount")
        if json_val.has("rxMsgCount"):
            self._rxMsgCount = json_val.getInt("rxMsgCount")
        if json_val.has("txMsgCount"):
            self._txMsgCount = json_val.getInt("txMsgCount")
        if json_val.has("lastMsg"):
            self._lastMsg = json_val.getString("lastMsg")
        if json_val.has("currentJob"):
            self._currentJob = json_val.getString("currentJob")
        if json_val.has("startupJob"):
            self._startupJob = json_val.getString("startupJob")
        if json_val.has("jobMaxTask"):
            self._jobMaxTask = json_val.getInt("jobMaxTask")
        if json_val.has("jobMaxSize"):
            self._jobMaxSize = json_val.getInt("jobMaxSize")
        if json_val.has("command"):
            self._command = json_val.getString("command")
        if json_val.has("protocol"):
            self._protocol = json_val.getString("protocol")
        if json_val.has("voltageLevel"):
            self._voltageLevel = json_val.getInt("voltageLevel")
        if json_val.has("serialMode"):
            self._serialMode = json_val.getString("serialMode")
        super(YSerialPort, self)._parseAttr(json_val)

    def get_rxCount(self):
        """
        Returns the total number of bytes received since last reset.

        @return an integer corresponding to the total number of bytes received since last reset

        On failure, throws an exception or returns YSerialPort.RXCOUNT_INVALID.
        """
        # res
        if self._cacheExpiration <= YAPI.GetTickCount():
            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                return YSerialPort.RXCOUNT_INVALID
        res = self._rxCount
        return res

    def get_txCount(self):
        """
        Returns the total number of bytes transmitted since last reset.

        @return an integer corresponding to the total number of bytes transmitted since last reset

        On failure, throws an exception or returns YSerialPort.TXCOUNT_INVALID.
        """
        # res
        if self._cacheExpiration <= YAPI.GetTickCount():
            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                return YSerialPort.TXCOUNT_INVALID
        res = self._txCount
        return res

    def get_errCount(self):
        """
        Returns the total number of communication errors detected since last reset.

        @return an integer corresponding to the total number of communication errors detected since last reset

        On failure, throws an exception or returns YSerialPort.ERRCOUNT_INVALID.
        """
        # res
        if self._cacheExpiration <= YAPI.GetTickCount():
            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                return YSerialPort.ERRCOUNT_INVALID
        res = self._errCount
        return res

    def get_rxMsgCount(self):
        """
        Returns the total number of messages received since last reset.

        @return an integer corresponding to the total number of messages received since last reset

        On failure, throws an exception or returns YSerialPort.RXMSGCOUNT_INVALID.
        """
        # res
        if self._cacheExpiration <= YAPI.GetTickCount():
            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                return YSerialPort.RXMSGCOUNT_INVALID
        res = self._rxMsgCount
        return res

    def get_txMsgCount(self):
        """
        Returns the total number of messages send since last reset.

        @return an integer corresponding to the total number of messages send since last reset

        On failure, throws an exception or returns YSerialPort.TXMSGCOUNT_INVALID.
        """
        # res
        if self._cacheExpiration <= YAPI.GetTickCount():
            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                return YSerialPort.TXMSGCOUNT_INVALID
        res = self._txMsgCount
        return res

    def get_lastMsg(self):
        """
        Returns the latest message fully received (for Line, Frame and Modbus protocols).

        @return a string corresponding to the latest message fully received (for Line, Frame and Modbus protocols)

        On failure, throws an exception or returns YSerialPort.LASTMSG_INVALID.
        """
        # res
        if self._cacheExpiration <= YAPI.GetTickCount():
            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                return YSerialPort.LASTMSG_INVALID
        res = self._lastMsg
        return res

    def get_currentJob(self):
        """
        Returns the name of the job file currently in use.

        @return a string corresponding to the name of the job file currently in use

        On failure, throws an exception or returns YSerialPort.CURRENTJOB_INVALID.
        """
        # res
        if self._cacheExpiration <= YAPI.GetTickCount():
            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                return YSerialPort.CURRENTJOB_INVALID
        res = self._currentJob
        return res

    def set_currentJob(self, newval):
        """
        Selects a job file to run immediately. If an empty string is
        given as argument, stops running current job file.

        @param newval : a string

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = newval
        return self._setAttr("currentJob", rest_val)

    def get_startupJob(self):
        """
        Returns the job file to use when the device is powered on.

        @return a string corresponding to the job file to use when the device is powered on

        On failure, throws an exception or returns YSerialPort.STARTUPJOB_INVALID.
        """
        # res
        if self._cacheExpiration <= YAPI.GetTickCount():
            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                return YSerialPort.STARTUPJOB_INVALID
        res = self._startupJob
        return res

    def set_startupJob(self, newval):
        """
        Changes the job to use when the device is powered on.
        Remember to call the saveToFlash() method of the module if the
        modification must be kept.

        @param newval : a string corresponding to the job to use when the device is powered on

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = newval
        return self._setAttr("startupJob", rest_val)

    def get_jobMaxTask(self):
        """
        Returns the maximum number of tasks in a job that the device can handle.

        @return an integer corresponding to the maximum number of tasks in a job that the device can handle

        On failure, throws an exception or returns YSerialPort.JOBMAXTASK_INVALID.
        """
        # res
        if self._cacheExpiration == datetime.datetime.fromtimestamp(86400):
            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                return YSerialPort.JOBMAXTASK_INVALID
        res = self._jobMaxTask
        return res

    def get_jobMaxSize(self):
        """
        Returns maximum size allowed for job files.

        @return an integer corresponding to maximum size allowed for job files

        On failure, throws an exception or returns YSerialPort.JOBMAXSIZE_INVALID.
        """
        # res
        if self._cacheExpiration == datetime.datetime.fromtimestamp(86400):
            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                return YSerialPort.JOBMAXSIZE_INVALID
        res = self._jobMaxSize
        return res

    def get_command(self):
        # res
        if self._cacheExpiration <= YAPI.GetTickCount():
            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                return YSerialPort.COMMAND_INVALID
        res = self._command
        return res

    def set_command(self, newval):
        rest_val = newval
        return self._setAttr("command", rest_val)

    def get_protocol(self):
        """
        Returns the type of protocol used over the serial line, as a string.
        Possible values are "Line" for ASCII messages separated by CR and/or LF,
        "StxEtx" for ASCII messages delimited by STX/ETX codes,
        "Frame:[timeout]ms" for binary messages separated by a delay time,
        "Modbus-ASCII" for MODBUS messages in ASCII mode,
        "Modbus-RTU" for MODBUS messages in RTU mode,
        "Wiegand-ASCII" for Wiegand messages in ASCII mode,
        "Wiegand-26","Wiegand-34", etc for Wiegand messages in byte mode,
        "Char" for a continuous ASCII stream or
        "Byte" for a continuous binary stream.

        @return a string corresponding to the type of protocol used over the serial line, as a string

        On failure, throws an exception or returns YSerialPort.PROTOCOL_INVALID.
        """
        # res
        if self._cacheExpiration <= YAPI.GetTickCount():
            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                return YSerialPort.PROTOCOL_INVALID
        res = self._protocol
        return res

    def set_protocol(self, newval):
        """
        Changes the type of protocol used over the serial line.
        Possible values are "Line" for ASCII messages separated by CR and/or LF,
        "StxEtx" for ASCII messages delimited by STX/ETX codes,
        "Frame:[timeout]ms" for binary messages separated by a delay time,
        "Modbus-ASCII" for MODBUS messages in ASCII mode,
        "Modbus-RTU" for MODBUS messages in RTU mode,
        "Wiegand-ASCII" for Wiegand messages in ASCII mode,
        "Wiegand-26","Wiegand-34", etc for Wiegand messages in byte mode,
        "Char" for a continuous ASCII stream or
        "Byte" for a continuous binary stream.
        The suffix "/[wait]ms" can be added to reduce the transmit rate so that there
        is always at lest the specified number of milliseconds between each bytes sent.
        Remember to call the saveToFlash() method of the module if the
        modification must be kept.

        @param newval : a string corresponding to the type of protocol used over the serial line

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = newval
        return self._setAttr("protocol", rest_val)

    def get_voltageLevel(self):
        """
        Returns the voltage level used on the serial line.

        @return a value among YSerialPort.VOLTAGELEVEL_OFF, YSerialPort.VOLTAGELEVEL_TTL3V,
        YSerialPort.VOLTAGELEVEL_TTL3VR, YSerialPort.VOLTAGELEVEL_TTL5V, YSerialPort.VOLTAGELEVEL_TTL5VR,
        YSerialPort.VOLTAGELEVEL_RS232, YSerialPort.VOLTAGELEVEL_RS485, YSerialPort.VOLTAGELEVEL_TTL1V8 and
        YSerialPort.VOLTAGELEVEL_SDI12 corresponding to the voltage level used on the serial line

        On failure, throws an exception or returns YSerialPort.VOLTAGELEVEL_INVALID.
        """
        # res
        if self._cacheExpiration <= YAPI.GetTickCount():
            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                return YSerialPort.VOLTAGELEVEL_INVALID
        res = self._voltageLevel
        return res

    def set_voltageLevel(self, newval):
        """
        Changes the voltage type used on the serial line. Valid
        values  will depend on the Yoctopuce device model featuring
        the serial port feature.  Check your device documentation
        to find out which values are valid for that specific model.
        Trying to set an invalid value will have no effect.
        Remember to call the saveToFlash() method of the module if the
        modification must be kept.

        @param newval : a value among YSerialPort.VOLTAGELEVEL_OFF, YSerialPort.VOLTAGELEVEL_TTL3V,
        YSerialPort.VOLTAGELEVEL_TTL3VR, YSerialPort.VOLTAGELEVEL_TTL5V, YSerialPort.VOLTAGELEVEL_TTL5VR,
        YSerialPort.VOLTAGELEVEL_RS232, YSerialPort.VOLTAGELEVEL_RS485, YSerialPort.VOLTAGELEVEL_TTL1V8 and
        YSerialPort.VOLTAGELEVEL_SDI12 corresponding to the voltage type used on the serial line

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = str(newval)
        return self._setAttr("voltageLevel", rest_val)

    def get_serialMode(self):
        """
        Returns the serial port communication parameters, as a string such as
        "9600,8N1". The string includes the baud rate, the number of data bits,
        the parity, and the number of stop bits. An optional suffix is included
        if flow control is active: "CtsRts" for hardware handshake, "XOnXOff"
        for logical flow control and "Simplex" for acquiring a shared bus using
        the RTS line (as used by some RS485 adapters for instance).

        @return a string corresponding to the serial port communication parameters, as a string such as
                "9600,8N1"

        On failure, throws an exception or returns YSerialPort.SERIALMODE_INVALID.
        """
        # res
        if self._cacheExpiration <= YAPI.GetTickCount():
            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                return YSerialPort.SERIALMODE_INVALID
        res = self._serialMode
        return res

    def set_serialMode(self, newval):
        """
        Changes the serial port communication parameters, with a string such as
        "9600,8N1". The string includes the baud rate, the number of data bits,
        the parity, and the number of stop bits. An optional suffix can be added
        to enable flow control: "CtsRts" for hardware handshake, "XOnXOff"
        for logical flow control and "Simplex" for acquiring a shared bus using
        the RTS line (as used by some RS485 adapters for instance).
        Remember to call the saveToFlash() method of the module if the
        modification must be kept.

        @param newval : a string corresponding to the serial port communication parameters, with a string such as
                "9600,8N1"

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = newval
        return self._setAttr("serialMode", rest_val)

    @staticmethod
    def FindSerialPort(func):
        """
        Retrieves a serial port for a given identifier.
        The identifier can be specified using several formats:
        <ul>
        <li>FunctionLogicalName</li>
        <li>ModuleSerialNumber.FunctionIdentifier</li>
        <li>ModuleSerialNumber.FunctionLogicalName</li>
        <li>ModuleLogicalName.FunctionIdentifier</li>
        <li>ModuleLogicalName.FunctionLogicalName</li>
        </ul>

        This function does not require that the serial port is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YSerialPort.isOnline() to test if the serial port is
        indeed online at a given time. In case of ambiguity when looking for
        a serial port by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        If a call to this object's is_online() method returns FALSE although
        you are certain that the matching device is plugged, make sure that you did
        call registerHub() at application initialization time.

        @param func : a string that uniquely characterizes the serial port, for instance
                RS232MK1.serialPort.

        @return a YSerialPort object allowing you to drive the serial port.
        """
        # obj
        obj = YFunction._FindFromCache("SerialPort", func)
        if obj is None:
            obj = YSerialPort(func)
            YFunction._AddToCache("SerialPort", func, obj)
        return obj

    def sendCommand(self, text):
        return self.set_command(text)

    def readLine(self):
        """
        Reads a single line (or message) from the receive buffer, starting at current stream position.
        This function is intended to be used when the serial port is configured for a message protocol,
        such as 'Line' mode or frame protocols.

        If data at current stream position is not available anymore in the receive buffer,
        the function returns the oldest available line and moves the stream position just after.
        If no new full line is received, the function returns an empty line.

        @return a string with a single line of text

        On failure, throws an exception or returns a negative error code.
        """
        # url
        # msgbin
        msgarr = []
        # msglen
        # res

        url = "rxmsg.json?pos=" + str(int(self._rxptr)) + "&len=1&maxw=1"
        msgbin = self._download(url)
        msgarr = self._json_get_array(msgbin)
        msglen = len(msgarr)
        if msglen == 0:
            return ""
        # // last element of array is the new position
        msglen = msglen - 1
        self._rxptr = YAPI._atoi(msgarr[msglen])
        if msglen == 0:
            return ""
        res = self._json_get_string(YString2Byte(msgarr[0]))
        return res

    def readMessages(self, pattern, maxWait):
        """
        Searches for incoming messages in the serial port receive buffer matching a given pattern,
        starting at current position. This function will only compare and return printable characters
        in the message strings. Binary protocols are handled as hexadecimal strings.

        The search returns all messages matching the expression provided as argument in the buffer.
        If no matching message is found, the search waits for one up to the specified maximum timeout
        (in milliseconds).

        @param pattern : a limited regular expression describing the expected message format,
                or an empty string if all messages should be returned (no filtering).
                When using binary protocols, the format applies to the hexadecimal
                representation of the message.
        @param maxWait : the maximum number of milliseconds to wait for a message if none is found
                in the receive buffer.

        @return an array of strings containing the messages found, if any.
                Binary messages are converted to hexadecimal representation.

        On failure, throws an exception or returns an empty array.
        """
        # url
        # msgbin
        msgarr = []
        # msglen
        res = []
        # idx

        url = "rxmsg.json?pos=" + str(int(self._rxptr)) + "&maxw=" + str(int(maxWait)) + "&pat=" + pattern
        msgbin = self._download(url)
        msgarr = self._json_get_array(msgbin)
        msglen = len(msgarr)
        if msglen == 0:
            return res
        # // last element of array is the new position
        msglen = msglen - 1
        self._rxptr = YAPI._atoi(msgarr[msglen])
        idx = 0

        while idx < msglen:
            res.append(self._json_get_string(YString2Byte(msgarr[idx])))
            idx = idx + 1

        return res

    def read_seek(self, absPos):
        """
        Changes the current internal stream position to the specified value. This function
        does not affect the device, it only changes the value stored in the API object
        for the next read operations.

        @param absPos : the absolute position index for next read operations.

        @return nothing.
        """
        self._rxptr = absPos
        return YAPI.SUCCESS

    def read_tell(self):
        """
        Returns the current absolute stream position pointer of the API object.

        @return the absolute position index for next read operations.
        """
        return self._rxptr

    def read_avail(self):
        """
        Returns the number of bytes available to read in the input buffer starting from the
        current absolute stream position pointer of the API object.

        @return the number of bytes available to read
        """
        # availPosStr
        # atPos
        # res
        # databin

        databin = self._download("rxcnt.bin?pos=" + str(int(self._rxptr)))
        availPosStr = YByte2String(databin)
        atPos = availPosStr.find("@")
        res = YAPI._atoi((availPosStr)[0: 0 + atPos])
        return res

    def end_tell(self):
        # availPosStr
        # atPos
        # res
        # databin

        databin = self._download("rxcnt.bin?pos=" + str(int(self._rxptr)))
        availPosStr = YByte2String(databin)
        atPos = availPosStr.find("@")
        res = YAPI._atoi((availPosStr)[atPos+1: atPos+1 + len(availPosStr)-atPos-1])
        return res

    def queryLine(self, query, maxWait):
        """
        Sends a text line query to the serial port, and reads the reply, if any.
        This function is intended to be used when the serial port is configured for 'Line' protocol.

        @param query : the line query to send (without CR/LF)
        @param maxWait : the maximum number of milliseconds to wait for a reply.

        @return the next text line received after sending the text query, as a string.
                Additional lines can be obtained by calling readLine or readMessages.

        On failure, throws an exception or returns an empty string.
        """
        # prevpos
        # url
        # msgbin
        msgarr = []
        # msglen
        # res
        if len(query) <= 80:
            # // fast query
            url = "rxmsg.json?len=1&maxw=" + str(int(maxWait)) + "&cmd=!" + self._escapeAttr(query)
        else:
            # // long query
            prevpos = self.end_tell()
            self._upload("txdata", YString2Byte(query + "\r\n"))
            url = "rxmsg.json?len=1&maxw=" + str(int(maxWait)) + "&pos=" + str(int(prevpos))

        msgbin = self._download(url)
        msgarr = self._json_get_array(msgbin)
        msglen = len(msgarr)
        if msglen == 0:
            return ""
        # // last element of array is the new position
        msglen = msglen - 1
        self._rxptr = YAPI._atoi(msgarr[msglen])
        if msglen == 0:
            return ""
        res = self._json_get_string(YString2Byte(msgarr[0]))
        return res

    def queryHex(self, hexString, maxWait):
        """
        Sends a binary message to the serial port, and reads the reply, if any.
        This function is intended to be used when the serial port is configured for
        Frame-based protocol.

        @param hexString : the message to send, coded in hexadecimal
        @param maxWait : the maximum number of milliseconds to wait for a reply.

        @return the next frame received after sending the message, as a hex string.
                Additional frames can be obtained by calling readHex or readMessages.

        On failure, throws an exception or returns an empty string.
        """
        # prevpos
        # url
        # msgbin
        msgarr = []
        # msglen
        # res
        if len(hexString) <= 80:
            # // fast query
            url = "rxmsg.json?len=1&maxw=" + str(int(maxWait)) + "&cmd=$" + hexString
        else:
            # // long query
            prevpos = self.end_tell()
            self._upload("txdata", YAPI._hexStrToBin(hexString))
            url = "rxmsg.json?len=1&maxw=" + str(int(maxWait)) + "&pos=" + str(int(prevpos))

        msgbin = self._download(url)
        msgarr = self._json_get_array(msgbin)
        msglen = len(msgarr)
        if msglen == 0:
            return ""
        # // last element of array is the new position
        msglen = msglen - 1
        self._rxptr = YAPI._atoi(msgarr[msglen])
        if msglen == 0:
            return ""
        res = self._json_get_string(YString2Byte(msgarr[0]))
        return res

    def uploadJob(self, jobfile, jsonDef):
        """
        Saves the job definition string (JSON data) into a job file.
        The job file can be later enabled using selectJob().

        @param jobfile : name of the job file to save on the device filesystem
        @param jsonDef : a string containing a JSON definition of the job

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        self._upload(jobfile, YString2Byte(jsonDef))
        return YAPI.SUCCESS

    def selectJob(self, jobfile):
        """
        Load and start processing the specified job file. The file must have
        been previously created using the user interface or uploaded on the
        device filesystem using the uploadJob() function.

        @param jobfile : name of the job file (on the device filesystem)

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        return self.set_currentJob(jobfile)

    def reset(self):
        """
        Clears the serial port buffer and resets counters to zero.

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        self._eventPos = 0
        self._rxptr = 0
        self._rxbuffptr = 0
        self._rxbuff = bytearray(0)

        return self.sendCommand("Z")

    def writeByte(self, code):
        """
        Sends a single byte to the serial port.

        @param code : the byte to send

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        return self.sendCommand("$" + ("%02X" % code))

    def writeStr(self, text):
        """
        Sends an ASCII string to the serial port, as is.

        @param text : the text string to send

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        # buff
        # bufflen
        # idx
        # ch
        buff = YString2Byte(text)
        bufflen = len(buff)
        if bufflen < 100:
            # // if string is pure text, we can send it as a simple command (faster)
            ch = 0x20
            idx = 0
            while (idx < bufflen) and (ch != 0):
                ch = YGetByte(buff, idx)
                if (ch >= 0x20) and (ch < 0x7f):
                    idx = idx + 1
                else:
                    ch = 0
            if idx >= bufflen:
                return self.sendCommand("+" + text)
        # // send string using file upload
        return self._upload("txdata", buff)

    def writeBin(self, buff):
        """
        Sends a binary buffer to the serial port, as is.

        @param buff : the binary buffer to send

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        return self._upload("txdata", buff)

    def writeArray(self, byteList):
        """
        Sends a byte sequence (provided as a list of bytes) to the serial port.

        @param byteList : a list of byte codes

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        # buff
        # bufflen
        # idx
        # hexb
        # res
        bufflen = len(byteList)
        buff = bytearray(bufflen)
        idx = 0
        while idx < bufflen:
            hexb = byteList[idx]
            buff[idx] = hexb
            idx = idx + 1

        res = self._upload("txdata", buff)
        return res

    def writeHex(self, hexString):
        """
        Sends a byte sequence (provided as a hexadecimal string) to the serial port.

        @param hexString : a string of hexadecimal byte codes

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        # buff
        # bufflen
        # idx
        # hexb
        # res
        bufflen = len(hexString)
        if bufflen < 100:
            return self.sendCommand("$" + hexString)
        bufflen = ((bufflen) >> (1))
        buff = bytearray(bufflen)
        idx = 0
        while idx < bufflen:
            hexb = int((hexString)[2 * idx: 2 * idx + 2], 16)
            buff[idx] = hexb
            idx = idx + 1

        res = self._upload("txdata", buff)
        return res

    def writeLine(self, text):
        """
        Sends an ASCII string to the serial port, followed by a line break (CR LF).

        @param text : the text string to send

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        # buff
        # bufflen
        # idx
        # ch
        buff = YString2Byte("" + text + "\r\n")
        bufflen = len(buff)-2
        if bufflen < 100:
            # // if string is pure text, we can send it as a simple command (faster)
            ch = 0x20
            idx = 0
            while (idx < bufflen) and (ch != 0):
                ch = YGetByte(buff, idx)
                if (ch >= 0x20) and (ch < 0x7f):
                    idx = idx + 1
                else:
                    ch = 0
            if idx >= bufflen:
                return self.sendCommand("!" + text)
        # // send string using file upload
        return self._upload("txdata", buff)

    def readByte(self):
        """
        Reads one byte from the receive buffer, starting at current stream position.
        If data at current stream position is not available anymore in the receive buffer,
        or if there is no data available yet, the function returns YAPI.NO_MORE_DATA.

        @return the next byte

        On failure, throws an exception or returns a negative error code.
        """
        # currpos
        # reqlen
        # buff
        # bufflen
        # mult
        # endpos
        # res
        # // first check if we have the requested character in the look-ahead buffer
        bufflen = len(self._rxbuff)
        if (self._rxptr >= self._rxbuffptr) and (self._rxptr < self._rxbuffptr+bufflen):
            res = YGetByte(self._rxbuff, self._rxptr-self._rxbuffptr)
            self._rxptr = self._rxptr + 1
            return res
        # // try to preload more than one byte to speed-up byte-per-byte access
        currpos = self._rxptr
        reqlen = 1024
        buff = self.readBin(reqlen)
        bufflen = len(buff)
        if self._rxptr == currpos+bufflen:
            res = YGetByte(buff, 0)
            self._rxptr = currpos+1
            self._rxbuffptr = currpos
            self._rxbuff = buff
            return res
        # // mixed bidirectional data, retry with a smaller block
        self._rxptr = currpos
        reqlen = 16
        buff = self.readBin(reqlen)
        bufflen = len(buff)
        if self._rxptr == currpos+bufflen:
            res = YGetByte(buff, 0)
            self._rxptr = currpos+1
            self._rxbuffptr = currpos
            self._rxbuff = buff
            return res
        # // still mixed, need to process character by character
        self._rxptr = currpos

        buff = self._download("rxdata.bin?pos=" + str(int(self._rxptr)) + "&len=1")
        bufflen = len(buff) - 1
        endpos = 0
        mult = 1
        while (bufflen > 0) and (YGetByte(buff, bufflen) != 64):
            endpos = endpos + mult * (YGetByte(buff, bufflen) - 48)
            mult = mult * 10
            bufflen = bufflen - 1
        self._rxptr = endpos
        if bufflen == 0:
            return YAPI.NO_MORE_DATA
        res = YGetByte(buff, 0)
        return res

    def readStr(self, nChars):
        """
        Reads data from the receive buffer as a string, starting at current stream position.
        If data at current stream position is not available anymore in the receive buffer, the
        function performs a short read.

        @param nChars : the maximum number of characters to read

        @return a string with receive buffer contents

        On failure, throws an exception or returns a negative error code.
        """
        # buff
        # bufflen
        # mult
        # endpos
        # res
        if nChars > 65535:
            nChars = 65535

        buff = self._download("rxdata.bin?pos=" + str(int(self._rxptr)) + "&len=" + str(int(nChars)))
        bufflen = len(buff) - 1
        endpos = 0
        mult = 1
        while (bufflen > 0) and (YGetByte(buff, bufflen) != 64):
            endpos = endpos + mult * (YGetByte(buff, bufflen) - 48)
            mult = mult * 10
            bufflen = bufflen - 1
        self._rxptr = endpos
        res = (YByte2String(buff))[0: 0 + bufflen]
        return res

    def readBin(self, nChars):
        """
        Reads data from the receive buffer as a binary buffer, starting at current stream position.
        If data at current stream position is not available anymore in the receive buffer, the
        function performs a short read.

        @param nChars : the maximum number of bytes to read

        @return a binary object with receive buffer contents

        On failure, throws an exception or returns a negative error code.
        """
        # buff
        # bufflen
        # mult
        # endpos
        # idx
        # res
        if nChars > 65535:
            nChars = 65535

        buff = self._download("rxdata.bin?pos=" + str(int(self._rxptr)) + "&len=" + str(int(nChars)))
        bufflen = len(buff) - 1
        endpos = 0
        mult = 1
        while (bufflen > 0) and (YGetByte(buff, bufflen) != 64):
            endpos = endpos + mult * (YGetByte(buff, bufflen) - 48)
            mult = mult * 10
            bufflen = bufflen - 1
        self._rxptr = endpos
        res = bytearray(bufflen)
        idx = 0
        while idx < bufflen:
            res[idx] = YGetByte(buff, idx)
            idx = idx + 1
        return res

    def readArray(self, nChars):
        """
        Reads data from the receive buffer as a list of bytes, starting at current stream position.
        If data at current stream position is not available anymore in the receive buffer, the
        function performs a short read.

        @param nChars : the maximum number of bytes to read

        @return a sequence of bytes with receive buffer contents

        On failure, throws an exception or returns an empty array.
        """
        # buff
        # bufflen
        # mult
        # endpos
        # idx
        # b
        res = []
        if nChars > 65535:
            nChars = 65535

        buff = self._download("rxdata.bin?pos=" + str(int(self._rxptr)) + "&len=" + str(int(nChars)))
        bufflen = len(buff) - 1
        endpos = 0
        mult = 1
        while (bufflen > 0) and (YGetByte(buff, bufflen) != 64):
            endpos = endpos + mult * (YGetByte(buff, bufflen) - 48)
            mult = mult * 10
            bufflen = bufflen - 1
        self._rxptr = endpos
        del res[:]
        idx = 0
        while idx < bufflen:
            b = YGetByte(buff, idx)
            res.append(b)
            idx = idx + 1

        return res

    def readHex(self, nBytes):
        """
        Reads data from the receive buffer as a hexadecimal string, starting at current stream position.
        If data at current stream position is not available anymore in the receive buffer, the
        function performs a short read.

        @param nBytes : the maximum number of bytes to read

        @return a string with receive buffer contents, encoded in hexadecimal

        On failure, throws an exception or returns a negative error code.
        """
        # buff
        # bufflen
        # mult
        # endpos
        # ofs
        # res
        if nBytes > 65535:
            nBytes = 65535

        buff = self._download("rxdata.bin?pos=" + str(int(self._rxptr)) + "&len=" + str(int(nBytes)))
        bufflen = len(buff) - 1
        endpos = 0
        mult = 1
        while (bufflen > 0) and (YGetByte(buff, bufflen) != 64):
            endpos = endpos + mult * (YGetByte(buff, bufflen) - 48)
            mult = mult * 10
            bufflen = bufflen - 1
        self._rxptr = endpos
        res = ""
        ofs = 0
        while ofs + 3 < bufflen:
            res = "" + res + "" + ("%02X" % YGetByte(buff, ofs)) + "" + ("%02X" % YGetByte(buff, ofs + 1)) + "" + ("%02X" % YGetByte(buff, ofs + 2)) + "" + ("%02X" % YGetByte(buff, ofs + 3))
            ofs = ofs + 4
        while ofs < bufflen:
            res = "" + res + "" + ("%02X" % YGetByte(buff, ofs))
            ofs = ofs + 1
        return res

    def sendBreak(self, duration):
        """
        Emits a BREAK condition on the serial interface. When the specified
        duration is 0, the BREAK signal will be exactly one character wide.
        When the duration is between 1 and 100, the BREAK condition will
        be hold for the specified number of milliseconds.

        @param duration : 0 for a standard BREAK, or duration between 1 and 100 ms

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        return self.sendCommand("B" + str(int(duration)))

    def set_RTS(self, val):
        """
        Manually sets the state of the RTS line. This function has no effect when
        hardware handshake is enabled, as the RTS line is driven automatically.

        @param val : 1 to turn RTS on, 0 to turn RTS off

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        return self.sendCommand("R" + str(int(val)))

    def get_CTS(self):
        """
        Reads the level of the CTS line. The CTS line is usually driven by
        the RTS signal of the connected serial device.

        @return 1 if the CTS line is high, 0 if the CTS line is low.

        On failure, throws an exception or returns a negative error code.
        """
        # buff
        # res

        buff = self._download("cts.txt")
        if not (len(buff) == 1):
            self._throw(YAPI.IO_ERROR, "invalid CTS reply")
            return YAPI.IO_ERROR
        res = YGetByte(buff, 0) - 48
        return res

    def snoopMessages(self, maxWait):
        """
        Retrieves messages (both direction) in the serial port buffer, starting at current position.
        This function will only compare and return printable characters in the message strings.
        Binary protocols are handled as hexadecimal strings.

        If no message is found, the search waits for one up to the specified maximum timeout
        (in milliseconds).

        @param maxWait : the maximum number of milliseconds to wait for a message if none is found
                in the receive buffer.

        @return an array of YSnoopingRecord objects containing the messages found, if any.
                Binary messages are converted to hexadecimal representation.

        On failure, throws an exception or returns an empty array.
        """
        # url
        # msgbin
        msgarr = []
        # msglen
        res = []
        # idx

        url = "rxmsg.json?pos=" + str(int(self._rxptr)) + "&maxw=" + str(int(maxWait)) + "&t=0"
        msgbin = self._download(url)
        msgarr = self._json_get_array(msgbin)
        msglen = len(msgarr)
        if msglen == 0:
            return res
        # // last element of array is the new position
        msglen = msglen - 1
        self._rxptr = YAPI._atoi(msgarr[msglen])
        idx = 0

        while idx < msglen:
            res.append(YSnoopingRecord(msgarr[idx]))
            idx = idx + 1

        return res

    def registerSnoopingCallback(self, callback):
        """
        Registers a callback function to be called each time that a message is sent or
        received by the serial port. The callback is invoked only during the execution of
        ySleep or yHandleEvents. This provides control over the time when
        the callback is triggered. For good responsiveness, remember to call one of these
        two functions periodically. To unregister a callback, pass a None pointer as argument.

        @param callback : the callback function to call, or a None pointer.
                The callback function should take four arguments:
                the YSerialPort object that emitted the event, and
                the YSnoopingRecord object that describes the message
                sent or received.
                On failure, throws an exception or returns a negative error code.
        """
        if callback is not None:
            self.registerValueCallback(yInternalEventCallback)
        else:
            self.registerValueCallback(None)
        # // register user callback AFTER the internal pseudo-event,
        # // to make sure we start with future events only
        self._eventCallback = callback
        return 0

    def _internalEventHandler(self, advstr):
        # url
        # msgbin
        msgarr = []
        # msglen
        # idx
        if not (self._eventCallback is not None):
            # // first simulated event, use it only to initialize reference values
            self._eventPos = 0

        url = "rxmsg.json?pos=" + str(int(self._eventPos)) + "&maxw=0&t=0"
        msgbin = self._download(url)
        msgarr = self._json_get_array(msgbin)
        msglen = len(msgarr)
        if msglen == 0:
            return YAPI.SUCCESS
        # // last element of array is the new position
        msglen = msglen - 1
        if not (self._eventCallback is not None):
            # // first simulated event, use it only to initialize reference values
            self._eventPos = YAPI._atoi(msgarr[msglen])
            return YAPI.SUCCESS
        self._eventPos = YAPI._atoi(msgarr[msglen])
        idx = 0
        while idx < msglen:
            self._eventCallback(self, YSnoopingRecord(msgarr[idx]))
            idx = idx + 1
        return YAPI.SUCCESS

    def writeStxEtx(self, text):
        """
        Sends an ASCII string to the serial port, preceeded with an STX code and
        followed by an ETX code.

        @param text : the text string to send

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        # buff
        buff = YString2Byte("" + str(chr(2)) + "" + text + "" + str(chr(3)))
        # // send string using file upload
        return self._upload("txdata", buff)

    def writeMODBUS(self, hexString):
        """
        Sends a MODBUS message (provided as a hexadecimal string) to the serial port.
        The message must start with the slave address. The MODBUS CRC/LRC is
        automatically added by the function. This function does not wait for a reply.

        @param hexString : a hexadecimal message string, including device address but no CRC/LRC

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        return self.sendCommand(":" + hexString)

    def queryMODBUS(self, slaveNo, pduBytes):
        """
        Sends a message to a specified MODBUS slave connected to the serial port, and reads the
        reply, if any. The message is the PDU, provided as a vector of bytes.

        @param slaveNo : the address of the slave MODBUS device to query
        @param pduBytes : the message to send (PDU), as a vector of bytes. The first byte of the
                PDU is the MODBUS function code.

        @return the received reply, as a vector of bytes.

        On failure, throws an exception or returns an empty array (or a MODBUS error reply).
        """
        # funCode
        # nib
        # i
        # cmd
        # prevpos
        # url
        # pat
        # msgs
        reps = []
        # rep
        res = []
        # replen
        # hexb
        funCode = pduBytes[0]
        nib = ((funCode) >> (4))
        pat = "" + ("%02X" % slaveNo) + "[" + ("%X" % nib) + "" + ("%X" % (nib+8)) + "]" + ("%X" % ((funCode) & (15))) + ".*"
        cmd = "" + ("%02X" % slaveNo) + "" + ("%02X" % funCode)
        i = 1
        while i < len(pduBytes):
            cmd = "" + cmd + "" + ("%02X" % ((pduBytes[i]) & (0xff)))
            i = i + 1
        if len(cmd) <= 80:
            # // fast query
            url = "rxmsg.json?cmd=:" + cmd + "&pat=:" + pat
        else:
            # // long query
            prevpos = self.end_tell()
            self._upload("txdata:", YAPI._hexStrToBin(cmd))
            url = "rxmsg.json?pos=" + str(int(prevpos)) + "&maxw=2000&pat=:" + pat

        msgs = self._download(url)
        reps = self._json_get_array(msgs)
        if not (len(reps) > 1):
            self._throw(YAPI.IO_ERROR, "no reply from MODBUS slave")
            return res
        if len(reps) > 1:
            rep = self._json_get_string(YString2Byte(reps[0]))
            replen = ((len(rep) - 3) >> (1))
            i = 0
            while i < replen:
                hexb = int((rep)[2 * i + 3: 2 * i + 3 + 2], 16)
                res.append(hexb)
                i = i + 1
            if res[0] != funCode:
                i = res[1]
                if not (i > 1):
                    self._throw(YAPI.NOT_SUPPORTED, "MODBUS error: unsupported function code")
                    return res
                if not (i > 2):
                    self._throw(YAPI.INVALID_ARGUMENT, "MODBUS error: illegal data address")
                    return res
                if not (i > 3):
                    self._throw(YAPI.INVALID_ARGUMENT, "MODBUS error: illegal data value")
                    return res
                if not (i > 4):
                    self._throw(YAPI.INVALID_ARGUMENT, "MODBUS error: failed to execute function")
                    return res
        return res

    def modbusReadBits(self, slaveNo, pduAddr, nBits):
        """
        Reads one or more contiguous internal bits (or coil status) from a MODBUS serial device.
        This method uses the MODBUS function code 0x01 (Read Coils).

        @param slaveNo : the address of the slave MODBUS device to query
        @param pduAddr : the relative address of the first bit/coil to read (zero-based)
        @param nBits : the number of bits/coils to read

        @return a vector of integers, each corresponding to one bit.

        On failure, throws an exception or returns an empty array.
        """
        pdu = []
        reply = []
        res = []
        # bitpos
        # idx
        # val
        # mask

        pdu.append(0x01)
        pdu.append(((pduAddr) >> (8)))
        pdu.append(((pduAddr) & (0xff)))
        pdu.append(((nBits) >> (8)))
        pdu.append(((nBits) & (0xff)))


        reply = self.queryMODBUS(slaveNo, pdu)
        if len(reply) == 0:
            return res
        if reply[0] != pdu[0]:
            return res

        bitpos = 0
        idx = 2
        val = reply[idx]
        mask = 1
        while bitpos < nBits:
            if ((val) & (mask)) == 0:
                res.append(0)
            else:
                res.append(1)
            bitpos = bitpos + 1
            if mask == 0x80:
                idx = idx + 1
                val = reply[idx]
                mask = 1
            else:
                mask = ((mask) << (1))

        return res

    def modbusReadInputBits(self, slaveNo, pduAddr, nBits):
        """
        Reads one or more contiguous input bits (or discrete inputs) from a MODBUS serial device.
        This method uses the MODBUS function code 0x02 (Read Discrete Inputs).

        @param slaveNo : the address of the slave MODBUS device to query
        @param pduAddr : the relative address of the first bit/input to read (zero-based)
        @param nBits : the number of bits/inputs to read

        @return a vector of integers, each corresponding to one bit.

        On failure, throws an exception or returns an empty array.
        """
        pdu = []
        reply = []
        res = []
        # bitpos
        # idx
        # val
        # mask

        pdu.append(0x02)
        pdu.append(((pduAddr) >> (8)))
        pdu.append(((pduAddr) & (0xff)))
        pdu.append(((nBits) >> (8)))
        pdu.append(((nBits) & (0xff)))


        reply = self.queryMODBUS(slaveNo, pdu)
        if len(reply) == 0:
            return res
        if reply[0] != pdu[0]:
            return res

        bitpos = 0
        idx = 2
        val = reply[idx]
        mask = 1
        while bitpos < nBits:
            if ((val) & (mask)) == 0:
                res.append(0)
            else:
                res.append(1)
            bitpos = bitpos + 1
            if mask == 0x80:
                idx = idx + 1
                val = reply[idx]
                mask = 1
            else:
                mask = ((mask) << (1))

        return res

    def modbusReadRegisters(self, slaveNo, pduAddr, nWords):
        """
        Reads one or more contiguous internal registers (holding registers) from a MODBUS serial device.
        This method uses the MODBUS function code 0x03 (Read Holding Registers).

        @param slaveNo : the address of the slave MODBUS device to query
        @param pduAddr : the relative address of the first holding register to read (zero-based)
        @param nWords : the number of holding registers to read

        @return a vector of integers, each corresponding to one 16-bit register value.

        On failure, throws an exception or returns an empty array.
        """
        pdu = []
        reply = []
        res = []
        # regpos
        # idx
        # val
        if not (nWords<=256):
            self._throw(YAPI.INVALID_ARGUMENT, "Cannot read more than 256 words")
            return res

        pdu.append(0x03)
        pdu.append(((pduAddr) >> (8)))
        pdu.append(((pduAddr) & (0xff)))
        pdu.append(((nWords) >> (8)))
        pdu.append(((nWords) & (0xff)))


        reply = self.queryMODBUS(slaveNo, pdu)
        if len(reply) == 0:
            return res
        if reply[0] != pdu[0]:
            return res

        regpos = 0
        idx = 2
        while regpos < nWords:
            val = ((reply[idx]) << (8))
            idx = idx + 1
            val = val + reply[idx]
            idx = idx + 1
            res.append(val)
            regpos = regpos + 1

        return res

    def modbusReadInputRegisters(self, slaveNo, pduAddr, nWords):
        """
        Reads one or more contiguous input registers (read-only registers) from a MODBUS serial device.
        This method uses the MODBUS function code 0x04 (Read Input Registers).

        @param slaveNo : the address of the slave MODBUS device to query
        @param pduAddr : the relative address of the first input register to read (zero-based)
        @param nWords : the number of input registers to read

        @return a vector of integers, each corresponding to one 16-bit input value.

        On failure, throws an exception or returns an empty array.
        """
        pdu = []
        reply = []
        res = []
        # regpos
        # idx
        # val

        pdu.append(0x04)
        pdu.append(((pduAddr) >> (8)))
        pdu.append(((pduAddr) & (0xff)))
        pdu.append(((nWords) >> (8)))
        pdu.append(((nWords) & (0xff)))


        reply = self.queryMODBUS(slaveNo, pdu)
        if len(reply) == 0:
            return res
        if reply[0] != pdu[0]:
            return res

        regpos = 0
        idx = 2
        while regpos < nWords:
            val = ((reply[idx]) << (8))
            idx = idx + 1
            val = val + reply[idx]
            idx = idx + 1
            res.append(val)
            regpos = regpos + 1

        return res

    def modbusWriteBit(self, slaveNo, pduAddr, value):
        """
        Sets a single internal bit (or coil) on a MODBUS serial device.
        This method uses the MODBUS function code 0x05 (Write Single Coil).

        @param slaveNo : the address of the slave MODBUS device to drive
        @param pduAddr : the relative address of the bit/coil to set (zero-based)
        @param value : the value to set (0 for OFF state, non-zero for ON state)

        @return the number of bits/coils affected on the device (1)

        On failure, throws an exception or returns zero.
        """
        pdu = []
        reply = []
        # res
        res = 0
        if value != 0:
            value = 0xff

        pdu.append(0x05)
        pdu.append(((pduAddr) >> (8)))
        pdu.append(((pduAddr) & (0xff)))
        pdu.append(value)
        pdu.append(0x00)


        reply = self.queryMODBUS(slaveNo, pdu)
        if len(reply) == 0:
            return res
        if reply[0] != pdu[0]:
            return res
        res = 1
        return res

    def modbusWriteBits(self, slaveNo, pduAddr, bits):
        """
        Sets several contiguous internal bits (or coils) on a MODBUS serial device.
        This method uses the MODBUS function code 0x0f (Write Multiple Coils).

        @param slaveNo : the address of the slave MODBUS device to drive
        @param pduAddr : the relative address of the first bit/coil to set (zero-based)
        @param bits : the vector of bits to be set (one integer per bit)

        @return the number of bits/coils affected on the device

        On failure, throws an exception or returns zero.
        """
        # nBits
        # nBytes
        # bitpos
        # val
        # mask
        pdu = []
        reply = []
        # res
        res = 0
        nBits = len(bits)
        nBytes = (((nBits + 7)) >> (3))

        pdu.append(0x0f)
        pdu.append(((pduAddr) >> (8)))
        pdu.append(((pduAddr) & (0xff)))
        pdu.append(((nBits) >> (8)))
        pdu.append(((nBits) & (0xff)))
        pdu.append(nBytes)
        bitpos = 0
        val = 0
        mask = 1
        while bitpos < nBits:
            if bits[bitpos] != 0:
                val = ((val) | (mask))
            bitpos = bitpos + 1
            if mask == 0x80:
                pdu.append(val)
                val = 0
                mask = 1
            else:
                mask = ((mask) << (1))
        if mask != 1:
            pdu.append(val)


        reply = self.queryMODBUS(slaveNo, pdu)
        if len(reply) == 0:
            return res
        if reply[0] != pdu[0]:
            return res
        res = ((reply[3]) << (8))
        res = res + reply[4]
        return res

    def modbusWriteRegister(self, slaveNo, pduAddr, value):
        """
        Sets a single internal register (or holding register) on a MODBUS serial device.
        This method uses the MODBUS function code 0x06 (Write Single Register).

        @param slaveNo : the address of the slave MODBUS device to drive
        @param pduAddr : the relative address of the register to set (zero-based)
        @param value : the 16 bit value to set

        @return the number of registers affected on the device (1)

        On failure, throws an exception or returns zero.
        """
        pdu = []
        reply = []
        # res
        res = 0

        pdu.append(0x06)
        pdu.append(((pduAddr) >> (8)))
        pdu.append(((pduAddr) & (0xff)))
        pdu.append(((value) >> (8)))
        pdu.append(((value) & (0xff)))


        reply = self.queryMODBUS(slaveNo, pdu)
        if len(reply) == 0:
            return res
        if reply[0] != pdu[0]:
            return res
        res = 1
        return res

    def modbusWriteRegisters(self, slaveNo, pduAddr, values):
        """
        Sets several contiguous internal registers (or holding registers) on a MODBUS serial device.
        This method uses the MODBUS function code 0x10 (Write Multiple Registers).

        @param slaveNo : the address of the slave MODBUS device to drive
        @param pduAddr : the relative address of the first internal register to set (zero-based)
        @param values : the vector of 16 bit values to set

        @return the number of registers affected on the device

        On failure, throws an exception or returns zero.
        """
        # nWords
        # nBytes
        # regpos
        # val
        pdu = []
        reply = []
        # res
        res = 0
        nWords = len(values)
        nBytes = 2 * nWords

        pdu.append(0x10)
        pdu.append(((pduAddr) >> (8)))
        pdu.append(((pduAddr) & (0xff)))
        pdu.append(((nWords) >> (8)))
        pdu.append(((nWords) & (0xff)))
        pdu.append(nBytes)
        regpos = 0
        while regpos < nWords:
            val = values[regpos]
            pdu.append(((val) >> (8)))
            pdu.append(((val) & (0xff)))
            regpos = regpos + 1


        reply = self.queryMODBUS(slaveNo, pdu)
        if len(reply) == 0:
            return res
        if reply[0] != pdu[0]:
            return res
        res = ((reply[3]) << (8))
        res = res + reply[4]
        return res

    def modbusWriteAndReadRegisters(self, slaveNo, pduWriteAddr, values, pduReadAddr, nReadWords):
        """
        Sets several contiguous internal registers (holding registers) on a MODBUS serial device,
        then performs a contiguous read of a set of (possibly different) internal registers.
        This method uses the MODBUS function code 0x17 (Read/Write Multiple Registers).

        @param slaveNo : the address of the slave MODBUS device to drive
        @param pduWriteAddr : the relative address of the first internal register to set (zero-based)
        @param values : the vector of 16 bit values to set
        @param pduReadAddr : the relative address of the first internal register to read (zero-based)
        @param nReadWords : the number of 16 bit values to read

        @return a vector of integers, each corresponding to one 16-bit register value read.

        On failure, throws an exception or returns an empty array.
        """
        # nWriteWords
        # nBytes
        # regpos
        # val
        # idx
        pdu = []
        reply = []
        res = []
        nWriteWords = len(values)
        nBytes = 2 * nWriteWords

        pdu.append(0x17)
        pdu.append(((pduReadAddr) >> (8)))
        pdu.append(((pduReadAddr) & (0xff)))
        pdu.append(((nReadWords) >> (8)))
        pdu.append(((nReadWords) & (0xff)))
        pdu.append(((pduWriteAddr) >> (8)))
        pdu.append(((pduWriteAddr) & (0xff)))
        pdu.append(((nWriteWords) >> (8)))
        pdu.append(((nWriteWords) & (0xff)))
        pdu.append(nBytes)
        regpos = 0
        while regpos < nWriteWords:
            val = values[regpos]
            pdu.append(((val) >> (8)))
            pdu.append(((val) & (0xff)))
            regpos = regpos + 1


        reply = self.queryMODBUS(slaveNo, pdu)
        if len(reply) == 0:
            return res
        if reply[0] != pdu[0]:
            return res

        regpos = 0
        idx = 2
        while regpos < nReadWords:
            val = ((reply[idx]) << (8))
            idx = idx + 1
            val = val + reply[idx]
            idx = idx + 1
            res.append(val)
            regpos = regpos + 1

        return res

    def nextSerialPort(self):
        """
        Continues the enumeration of serial ports started using yFirstSerialPort().
        Caution: You can't make any assumption about the returned serial ports order.
        If you want to find a specific a serial port, use SerialPort.findSerialPort()
        and a hardwareID or a logical name.

        @return a pointer to a YSerialPort object, corresponding to
                a serial port currently online, or a None pointer
                if there are no more serial ports to enumerate.
        """
        hwidRef = YRefParam()
        if YAPI.YISERR(self._nextFunction(hwidRef)):
            return None
        if hwidRef.value == "":
            return None
        return YSerialPort.FindSerialPort(hwidRef.value)

#--- (end of generated code: YSerialPort implementation)

#--- (generated code: YSerialPort functions)

    @staticmethod
    def FirstSerialPort():
        """
        Starts the enumeration of serial ports currently accessible.
        Use the method YSerialPort.nextSerialPort() to iterate on
        next serial ports.

        @return a pointer to a YSerialPort object, corresponding to
                the first serial port currently online, or a None pointer
                if there are none.
        """
        devRef = YRefParam()
        neededsizeRef = YRefParam()
        serialRef = YRefParam()
        funcIdRef = YRefParam()
        funcNameRef = YRefParam()
        funcValRef = YRefParam()
        errmsgRef = YRefParam()
        size = YAPI.C_INTSIZE
        #noinspection PyTypeChecker,PyCallingNonCallable
        p = (ctypes.c_int * 1)()
        err = YAPI.apiGetFunctionsByClass("SerialPort", 0, p, size, neededsizeRef, errmsgRef)

        if YAPI.YISERR(err) or not neededsizeRef.value:
            return None

        if YAPI.YISERR(
                YAPI.yapiGetFunctionInfo(p[0], devRef, serialRef, funcIdRef, funcNameRef, funcValRef, errmsgRef)):
            return None

        return YSerialPort.FindSerialPort(serialRef.value + "." + funcIdRef.value)

#--- (end of generated code: YSerialPort functions)



# -*- coding: utf-8 -*-
# ********************************************************************
#
#  $Id: yocto_powersupply.py 55633 2023-07-26 09:18:40Z seb $
#
#  Implements yFindPowerSupply(), the high-level API for PowerSupply functions
#
#  - - - - - - - - - License information: - - - - - - - - -
#
#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
#
#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
#  non-exclusive license to use, modify, copy and integrate this
#  file into your software for the sole purpose of interfacing
#  with Yoctopuce products.
#
#  You may reproduce and distribute copies of this file in
#  source or object form, as long as the sole purpose of this
#  code is to interface with Yoctopuce products. You must retain
#  this notice in the distributed source file.
#
#  You should refer to Yoctopuce General Terms and Conditions
#  for additional information regarding your rights and
#  obligations.
#
#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
#  WARRANTY, OR OTHERWISE.
#
# *********************************************************************


__docformat__ = 'restructuredtext en'
from yoctopuce.yocto_api import *


#--- (YPowerSupply class start)
#noinspection PyProtectedMember
class YPowerSupply(YFunction):
    """
    The YPowerSupply class allows you to drive a Yoctopuce power supply.
    It can be use to change the voltage and current limits, and to enable/disable
    the output.

    """
    #--- (end of YPowerSupply class start)
    #--- (YPowerSupply return codes)
    #--- (end of YPowerSupply return codes)
    #--- (YPowerSupply dlldef)
    #--- (end of YPowerSupply dlldef)
    #--- (YPowerSupply yapiwrapper)
    #--- (end of YPowerSupply yapiwrapper)
    #--- (YPowerSupply definitions)
    VOLTAGELIMIT_INVALID = YAPI.INVALID_DOUBLE
    CURRENTLIMIT_INVALID = YAPI.INVALID_DOUBLE
    MEASUREDVOLTAGE_INVALID = YAPI.INVALID_DOUBLE
    MEASUREDCURRENT_INVALID = YAPI.INVALID_DOUBLE
    INPUTVOLTAGE_INVALID = YAPI.INVALID_DOUBLE
    VOLTAGETRANSITION_INVALID = YAPI.INVALID_STRING
    VOLTAGELIMITATSTARTUP_INVALID = YAPI.INVALID_DOUBLE
    CURRENTLIMITATSTARTUP_INVALID = YAPI.INVALID_DOUBLE
    COMMAND_INVALID = YAPI.INVALID_STRING
    POWEROUTPUT_OFF = 0
    POWEROUTPUT_ON = 1
    POWEROUTPUT_INVALID = -1
    POWEROUTPUTATSTARTUP_OFF = 0
    POWEROUTPUTATSTARTUP_ON = 1
    POWEROUTPUTATSTARTUP_INVALID = -1
    #--- (end of YPowerSupply definitions)

    def __init__(self, func):
        super(YPowerSupply, self).__init__(func)
        self._className = 'PowerSupply'
        #--- (YPowerSupply attributes)
        self._callback = None
        self._voltageLimit = YPowerSupply.VOLTAGELIMIT_INVALID
        self._currentLimit = YPowerSupply.CURRENTLIMIT_INVALID
        self._powerOutput = YPowerSupply.POWEROUTPUT_INVALID
        self._measuredVoltage = YPowerSupply.MEASUREDVOLTAGE_INVALID
        self._measuredCurrent = YPowerSupply.MEASUREDCURRENT_INVALID
        self._inputVoltage = YPowerSupply.INPUTVOLTAGE_INVALID
        self._voltageTransition = YPowerSupply.VOLTAGETRANSITION_INVALID
        self._voltageLimitAtStartUp = YPowerSupply.VOLTAGELIMITATSTARTUP_INVALID
        self._currentLimitAtStartUp = YPowerSupply.CURRENTLIMITATSTARTUP_INVALID
        self._powerOutputAtStartUp = YPowerSupply.POWEROUTPUTATSTARTUP_INVALID
        self._command = YPowerSupply.COMMAND_INVALID
        #--- (end of YPowerSupply attributes)

    #--- (YPowerSupply implementation)
    def _parseAttr(self, json_val):
        if json_val.has("voltageLimit"):
            self._voltageLimit = round(json_val.getDouble("voltageLimit") / 65.536) / 1000.0
        if json_val.has("currentLimit"):
            self._currentLimit = round(json_val.getDouble("currentLimit") / 65.536) / 1000.0
        if json_val.has("powerOutput"):
            self._powerOutput = (json_val.getInt("powerOutput") > 0 if 1 else 0)
        if json_val.has("measuredVoltage"):
            self._measuredVoltage = round(json_val.getDouble("measuredVoltage") / 65.536) / 1000.0
        if json_val.has("measuredCurrent"):
            self._measuredCurrent = round(json_val.getDouble("measuredCurrent") / 65.536) / 1000.0
        if json_val.has("inputVoltage"):
            self._inputVoltage = round(json_val.getDouble("inputVoltage") / 65.536) / 1000.0
        if json_val.has("voltageTransition"):
            self._voltageTransition = json_val.getString("voltageTransition")
        if json_val.has("voltageLimitAtStartUp"):
            self._voltageLimitAtStartUp = round(json_val.getDouble("voltageLimitAtStartUp") / 65.536) / 1000.0
        if json_val.has("currentLimitAtStartUp"):
            self._currentLimitAtStartUp = round(json_val.getDouble("currentLimitAtStartUp") / 65.536) / 1000.0
        if json_val.has("powerOutputAtStartUp"):
            self._powerOutputAtStartUp = (json_val.getInt("powerOutputAtStartUp") > 0 if 1 else 0)
        if json_val.has("command"):
            self._command = json_val.getString("command")
        super(YPowerSupply, self)._parseAttr(json_val)

    def set_voltageLimit(self, newval):
        """
        Changes the voltage limit, in V.

        @param newval : a floating point number corresponding to the voltage limit, in V

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = str(int(round(newval * 65536.0, 1)))
        return self._setAttr("voltageLimit", rest_val)

    def get_voltageLimit(self):
        """
        Returns the voltage limit, in V.

        @return a floating point number corresponding to the voltage limit, in V

        On failure, throws an exception or returns YPowerSupply.VOLTAGELIMIT_INVALID.
        """
        # res
        if self._cacheExpiration <= YAPI.GetTickCount():
            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                return YPowerSupply.VOLTAGELIMIT_INVALID
        res = self._voltageLimit
        return res

    def set_currentLimit(self, newval):
        """
        Changes the current limit, in mA.

        @param newval : a floating point number corresponding to the current limit, in mA

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = str(int(round(newval * 65536.0, 1)))
        return self._setAttr("currentLimit", rest_val)

    def get_currentLimit(self):
        """
        Returns the current limit, in mA.

        @return a floating point number corresponding to the current limit, in mA

        On failure, throws an exception or returns YPowerSupply.CURRENTLIMIT_INVALID.
        """
        # res
        if self._cacheExpiration <= YAPI.GetTickCount():
            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                return YPowerSupply.CURRENTLIMIT_INVALID
        res = self._currentLimit
        return res

    def get_powerOutput(self):
        """
        Returns the power supply output switch state.

        @return either YPowerSupply.POWEROUTPUT_OFF or YPowerSupply.POWEROUTPUT_ON, according to the power
        supply output switch state

        On failure, throws an exception or returns YPowerSupply.POWEROUTPUT_INVALID.
        """
        # res
        if self._cacheExpiration <= YAPI.GetTickCount():
            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                return YPowerSupply.POWEROUTPUT_INVALID
        res = self._powerOutput
        return res

    def set_powerOutput(self, newval):
        """
        Changes the power supply output switch state.

        @param newval : either YPowerSupply.POWEROUTPUT_OFF or YPowerSupply.POWEROUTPUT_ON, according to
        the power supply output switch state

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = "1" if newval > 0 else "0"
        return self._setAttr("powerOutput", rest_val)

    def get_measuredVoltage(self):
        """
        Returns the measured output voltage, in V.

        @return a floating point number corresponding to the measured output voltage, in V

        On failure, throws an exception or returns YPowerSupply.MEASUREDVOLTAGE_INVALID.
        """
        # res
        if self._cacheExpiration <= YAPI.GetTickCount():
            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                return YPowerSupply.MEASUREDVOLTAGE_INVALID
        res = self._measuredVoltage
        return res

    def get_measuredCurrent(self):
        """
        Returns the measured output current, in mA.

        @return a floating point number corresponding to the measured output current, in mA

        On failure, throws an exception or returns YPowerSupply.MEASUREDCURRENT_INVALID.
        """
        # res
        if self._cacheExpiration <= YAPI.GetTickCount():
            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                return YPowerSupply.MEASUREDCURRENT_INVALID
        res = self._measuredCurrent
        return res

    def get_inputVoltage(self):
        """
        Returns the measured input voltage, in V.

        @return a floating point number corresponding to the measured input voltage, in V

        On failure, throws an exception or returns YPowerSupply.INPUTVOLTAGE_INVALID.
        """
        # res
        if self._cacheExpiration <= YAPI.GetTickCount():
            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                return YPowerSupply.INPUTVOLTAGE_INVALID
        res = self._inputVoltage
        return res

    def get_voltageTransition(self):
        # res
        if self._cacheExpiration <= YAPI.GetTickCount():
            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                return YPowerSupply.VOLTAGETRANSITION_INVALID
        res = self._voltageTransition
        return res

    def set_voltageTransition(self, newval):
        rest_val = newval
        return self._setAttr("voltageTransition", rest_val)

    def set_voltageLimitAtStartUp(self, newval):
        """
        Changes the voltage set point at device start up. Remember to call the matching
        module saveToFlash() method, otherwise this call has no effect.

        @param newval : a floating point number corresponding to the voltage set point at device start up

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = str(int(round(newval * 65536.0, 1)))
        return self._setAttr("voltageLimitAtStartUp", rest_val)

    def get_voltageLimitAtStartUp(self):
        """
        Returns the selected voltage limit at device startup, in V.

        @return a floating point number corresponding to the selected voltage limit at device startup, in V

        On failure, throws an exception or returns YPowerSupply.VOLTAGELIMITATSTARTUP_INVALID.
        """
        # res
        if self._cacheExpiration <= YAPI.GetTickCount():
            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                return YPowerSupply.VOLTAGELIMITATSTARTUP_INVALID
        res = self._voltageLimitAtStartUp
        return res

    def set_currentLimitAtStartUp(self, newval):
        """
        Changes the current limit at device start up. Remember to call the matching
        module saveToFlash() method, otherwise this call has no effect.

        @param newval : a floating point number corresponding to the current limit at device start up

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = str(int(round(newval * 65536.0, 1)))
        return self._setAttr("currentLimitAtStartUp", rest_val)

    def get_currentLimitAtStartUp(self):
        """
        Returns the selected current limit at device startup, in mA.

        @return a floating point number corresponding to the selected current limit at device startup, in mA

        On failure, throws an exception or returns YPowerSupply.CURRENTLIMITATSTARTUP_INVALID.
        """
        # res
        if self._cacheExpiration <= YAPI.GetTickCount():
            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                return YPowerSupply.CURRENTLIMITATSTARTUP_INVALID
        res = self._currentLimitAtStartUp
        return res

    def get_powerOutputAtStartUp(self):
        """
        Returns the power supply output switch state.

        @return either YPowerSupply.POWEROUTPUTATSTARTUP_OFF or YPowerSupply.POWEROUTPUTATSTARTUP_ON,
        according to the power supply output switch state

        On failure, throws an exception or returns YPowerSupply.POWEROUTPUTATSTARTUP_INVALID.
        """
        # res
        if self._cacheExpiration <= YAPI.GetTickCount():
            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                return YPowerSupply.POWEROUTPUTATSTARTUP_INVALID
        res = self._powerOutputAtStartUp
        return res

    def set_powerOutputAtStartUp(self, newval):
        """
        Changes the power supply output switch state at device start up. Remember to call the matching
        module saveToFlash() method, otherwise this call has no effect.

        @param newval : either YPowerSupply.POWEROUTPUTATSTARTUP_OFF or
        YPowerSupply.POWEROUTPUTATSTARTUP_ON, according to the power supply output switch state at device start up

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = "1" if newval > 0 else "0"
        return self._setAttr("powerOutputAtStartUp", rest_val)

    def get_command(self):
        # res
        if self._cacheExpiration <= YAPI.GetTickCount():
            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                return YPowerSupply.COMMAND_INVALID
        res = self._command
        return res

    def set_command(self, newval):
        rest_val = newval
        return self._setAttr("command", rest_val)

    @staticmethod
    def FindPowerSupply(func):
        """
        Retrieves a regulated power supply for a given identifier.
        The identifier can be specified using several formats:
        <ul>
        <li>FunctionLogicalName</li>
        <li>ModuleSerialNumber.FunctionIdentifier</li>
        <li>ModuleSerialNumber.FunctionLogicalName</li>
        <li>ModuleLogicalName.FunctionIdentifier</li>
        <li>ModuleLogicalName.FunctionLogicalName</li>
        </ul>

        This function does not require that the regulated power supply is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YPowerSupply.isOnline() to test if the regulated power supply is
        indeed online at a given time. In case of ambiguity when looking for
        a regulated power supply by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        If a call to this object's is_online() method returns FALSE although
        you are certain that the matching device is plugged, make sure that you did
        call registerHub() at application initialization time.

        @param func : a string that uniquely characterizes the regulated power supply, for instance
                MyDevice.powerSupply.

        @return a YPowerSupply object allowing you to drive the regulated power supply.
        """
        # obj
        obj = YFunction._FindFromCache("PowerSupply", func)
        if obj is None:
            obj = YPowerSupply(func)
            YFunction._AddToCache("PowerSupply", func, obj)
        return obj

    def voltageMove(self, V_target, ms_duration):
        """
        Performs a smooth transition of output voltage. Any explicit voltage
        change cancels any ongoing transition process.

        @param V_target   : new output voltage value at the end of the transition
                (floating-point number, representing the end voltage in V)
        @param ms_duration : total duration of the transition, in milliseconds

        @return YAPI.SUCCESS when the call succeeds.
        """
        # newval
        if V_target < 0.0:
            V_target  = 0.0
        newval = "" + str(int(round(V_target*65536))) + ":" + str(int(ms_duration))

        return self.set_voltageTransition(newval)

    def nextPowerSupply(self):
        """
        Continues the enumeration of regulated power supplies started using yFirstPowerSupply().
        Caution: You can't make any assumption about the returned regulated power supplies order.
        If you want to find a specific a regulated power supply, use PowerSupply.findPowerSupply()
        and a hardwareID or a logical name.

        @return a pointer to a YPowerSupply object, corresponding to
                a regulated power supply currently online, or a None pointer
                if there are no more regulated power supplies to enumerate.
        """
        hwidRef = YRefParam()
        if YAPI.YISERR(self._nextFunction(hwidRef)):
            return None
        if hwidRef.value == "":
            return None
        return YPowerSupply.FindPowerSupply(hwidRef.value)

#--- (end of YPowerSupply implementation)

#--- (YPowerSupply functions)

    @staticmethod
    def FirstPowerSupply():
        """
        Starts the enumeration of regulated power supplies currently accessible.
        Use the method YPowerSupply.nextPowerSupply() to iterate on
        next regulated power supplies.

        @return a pointer to a YPowerSupply object, corresponding to
                the first regulated power supply currently online, or a None pointer
                if there are none.
        """
        devRef = YRefParam()
        neededsizeRef = YRefParam()
        serialRef = YRefParam()
        funcIdRef = YRefParam()
        funcNameRef = YRefParam()
        funcValRef = YRefParam()
        errmsgRef = YRefParam()
        size = YAPI.C_INTSIZE
        #noinspection PyTypeChecker,PyCallingNonCallable
        p = (ctypes.c_int * 1)()
        err = YAPI.apiGetFunctionsByClass("PowerSupply", 0, p, size, neededsizeRef, errmsgRef)

        if YAPI.YISERR(err) or not neededsizeRef.value:
            return None

        if YAPI.YISERR(
                YAPI.yapiGetFunctionInfo(p[0], devRef, serialRef, funcIdRef, funcNameRef, funcValRef, errmsgRef)):
            return None

        return YPowerSupply.FindPowerSupply(serialRef.value + "." + funcIdRef.value)

#--- (end of YPowerSupply functions)



# -*- coding: utf-8 -*-
# ********************************************************************
#
#  $Id: yocto_poweroutput.py 38913 2019-12-20 18:59:49Z mvuilleu $
#
#  Implements yFindPowerOutput(), the high-level API for PowerOutput functions
#
#  - - - - - - - - - License information: - - - - - - - - -
#
#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
#
#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
#  non-exclusive license to use, modify, copy and integrate this
#  file into your software for the sole purpose of interfacing
#  with Yoctopuce products.
#
#  You may reproduce and distribute copies of this file in
#  source or object form, as long as the sole purpose of this
#  code is to interface with Yoctopuce products. You must retain
#  this notice in the distributed source file.
#
#  You should refer to Yoctopuce General Terms and Conditions
#  for additional information regarding your rights and
#  obligations.
#
#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
#  WARRANTY, OR OTHERWISE.
#
# *********************************************************************


__docformat__ = 'restructuredtext en'
from yoctopuce.yocto_api import *


#--- (YPowerOutput class start)
#noinspection PyProtectedMember
class YPowerOutput(YFunction):
    """
    The YPowerOutput class allows you to control
    the power output featured on some Yoctopuce devices.

    """
    #--- (end of YPowerOutput class start)
    #--- (YPowerOutput return codes)
    #--- (end of YPowerOutput return codes)
    #--- (YPowerOutput dlldef)
    #--- (end of YPowerOutput dlldef)
    #--- (YPowerOutput yapiwrapper)
    #--- (end of YPowerOutput yapiwrapper)
    #--- (YPowerOutput definitions)
    VOLTAGE_OFF = 0
    VOLTAGE_OUT3V3 = 1
    VOLTAGE_OUT5V = 2
    VOLTAGE_OUT4V7 = 3
    VOLTAGE_OUT1V8 = 4
    VOLTAGE_INVALID = -1
    #--- (end of YPowerOutput definitions)

    def __init__(self, func):
        super(YPowerOutput, self).__init__(func)
        self._className = 'PowerOutput'
        #--- (YPowerOutput attributes)
        self._callback = None
        self._voltage = YPowerOutput.VOLTAGE_INVALID
        #--- (end of YPowerOutput attributes)

    #--- (YPowerOutput implementation)
    def _parseAttr(self, json_val):
        if json_val.has("voltage"):
            self._voltage = json_val.getInt("voltage")
        super(YPowerOutput, self)._parseAttr(json_val)

    def get_voltage(self):
        """
        Returns the voltage on the power output featured by the module.

        @return a value among YPowerOutput.VOLTAGE_OFF, YPowerOutput.VOLTAGE_OUT3V3,
        YPowerOutput.VOLTAGE_OUT5V, YPowerOutput.VOLTAGE_OUT4V7 and YPowerOutput.VOLTAGE_OUT1V8
        corresponding to the voltage on the power output featured by the module

        On failure, throws an exception or returns YPowerOutput.VOLTAGE_INVALID.
        """
        # res
        if self._cacheExpiration <= YAPI.GetTickCount():
            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                return YPowerOutput.VOLTAGE_INVALID
        res = self._voltage
        return res

    def set_voltage(self, newval):
        """
        Changes the voltage on the power output provided by the
        module. Remember to call the saveToFlash() method of the module if the
        modification must be kept.

        @param newval : a value among YPowerOutput.VOLTAGE_OFF, YPowerOutput.VOLTAGE_OUT3V3,
        YPowerOutput.VOLTAGE_OUT5V, YPowerOutput.VOLTAGE_OUT4V7 and YPowerOutput.VOLTAGE_OUT1V8
        corresponding to the voltage on the power output provided by the
                module

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = str(newval)
        return self._setAttr("voltage", rest_val)

    @staticmethod
    def FindPowerOutput(func):
        """
        Retrieves a power output for a given identifier.
        The identifier can be specified using several formats:
        <ul>
        <li>FunctionLogicalName</li>
        <li>ModuleSerialNumber.FunctionIdentifier</li>
        <li>ModuleSerialNumber.FunctionLogicalName</li>
        <li>ModuleLogicalName.FunctionIdentifier</li>
        <li>ModuleLogicalName.FunctionLogicalName</li>
        </ul>

        This function does not require that the power output is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YPowerOutput.isOnline() to test if the power output is
        indeed online at a given time. In case of ambiguity when looking for
        a power output by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        If a call to this object's is_online() method returns FALSE although
        you are certain that the matching device is plugged, make sure that you did
        call registerHub() at application initialization time.

        @param func : a string that uniquely characterizes the power output, for instance
                YI2CMK01.powerOutput.

        @return a YPowerOutput object allowing you to drive the power output.
        """
        # obj
        obj = YFunction._FindFromCache("PowerOutput", func)
        if obj is None:
            obj = YPowerOutput(func)
            YFunction._AddToCache("PowerOutput", func, obj)
        return obj

    def nextPowerOutput(self):
        """
        Continues the enumeration of power output started using yFirstPowerOutput().
        Caution: You can't make any assumption about the returned power output order.
        If you want to find a specific a power output, use PowerOutput.findPowerOutput()
        and a hardwareID or a logical name.

        @return a pointer to a YPowerOutput object, corresponding to
                a power output currently online, or a None pointer
                if there are no more power output to enumerate.
        """
        hwidRef = YRefParam()
        if YAPI.YISERR(self._nextFunction(hwidRef)):
            return None
        if hwidRef.value == "":
            return None
        return YPowerOutput.FindPowerOutput(hwidRef.value)

#--- (end of YPowerOutput implementation)

#--- (YPowerOutput functions)

    @staticmethod
    def FirstPowerOutput():
        """
        Starts the enumeration of power output currently accessible.
        Use the method YPowerOutput.nextPowerOutput() to iterate on
        next power output.

        @return a pointer to a YPowerOutput object, corresponding to
                the first power output currently online, or a None pointer
                if there are none.
        """
        devRef = YRefParam()
        neededsizeRef = YRefParam()
        serialRef = YRefParam()
        funcIdRef = YRefParam()
        funcNameRef = YRefParam()
        funcValRef = YRefParam()
        errmsgRef = YRefParam()
        size = YAPI.C_INTSIZE
        #noinspection PyTypeChecker,PyCallingNonCallable
        p = (ctypes.c_int * 1)()
        err = YAPI.apiGetFunctionsByClass("PowerOutput", 0, p, size, neededsizeRef, errmsgRef)

        if YAPI.YISERR(err) or not neededsizeRef.value:
            return None

        if YAPI.YISERR(
                YAPI.yapiGetFunctionInfo(p[0], devRef, serialRef, funcIdRef, funcNameRef, funcValRef, errmsgRef)):
            return None

        return YPowerOutput.FindPowerOutput(serialRef.value + "." + funcIdRef.value)

#--- (end of YPowerOutput functions)



#!/usr/bin/python
# -*- coding: utf-8 -*-
# add ../../Sources to the PYTHONPATH
import sys
import os

sys.path.append(os.path.join("..", "..", "Sources"))
from yoctopuce.yocto_api import *
from yoctopuce.yocto_colorledcluster import *


def main():
    errmsg = YRefParam()
    # Setup the API to use local USB devices
    if YAPI.RegisterHub("usb", errmsg) != YAPI.SUCCESS:
        sys.exit("init error" + errmsg.value)

    leds = YColorLedCluster.FirstColorLedCluster()
    if leds is None:
        sys.exit("No Color Led cluster found (check USB cable)")

    leds.resetBlinkSeq(0)  # cleans the sequence 0
    leds.addRgbMoveToBlinkSeq(0, 0x400000, 0)    # move instantaneously to red 25%
    leds.addRgbMoveToBlinkSeq(0, 0x400000, 500)  # stay on the same color for 0.5sec
    leds.addRgbMoveToBlinkSeq(0, 0x000040, 0)    # move instantaneously to red 25%
    leds.addRgbMoveToBlinkSeq(0, 0x000040, 500)  # stay on the same color for 0.5sec
    leds.linkLedToBlinkSeq(0, 1, 0, 0)           # link led 0 to sequence 0
    leds.linkLedToBlinkSeqAtPowerOn(0, 1, 0, 0)  # led 0 will wutomatically be linked to sequence 0  at startup
    leds.set_blinkSeqStateAtPowerOn(0, 1)        # sequence 0  will automatically run at startup
    leds.saveBlinkSeq(0);                        # save the sequence on flash memory

    leds.resetBlinkSeq(1)  # cleans the sequence  1
    leds.addHslMoveToBlinkSeq(1, 0x00FF20, 2000)  # Circle over all hue values
    leds.addHslMoveToBlinkSeq(1, 0x55FF20, 2000)  # Sturation =100%
    leds.addHslMoveToBlinkSeq(1, 0xAAFF20, 2000)  # luminosiy = 12%
    leds.linkLedToBlinkSeq(1, 1, 1, 0)            # link led 1 to sequence 1
    leds.linkLedToBlinkSeqAtPowerOn(1, 1, 1, 0)   # led 1 will wutomatically be linked to sequence 1  at startup
    leds.set_blinkSeqStateAtPowerOn(1, 1)         #  sequence 1  will automatically run at startup
    leds.saveBlinkSeq(1)                          # save the sequence on flash  memory

    leds.saveLedsConfigAtPowerOn();               # All leds configuration

    leds.startBlinkSeq(0)                         # start sequence 0
    leds.startBlinkSeq(1)                         # start sequence 1

    print("The leds are now blinking autonomously");
    YAPI.FreeAPI()

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# ********************************************************************
#
#  $Id: svn_id $
#
#  Implements yFindInputCaptureData(), the high-level API for InputCaptureData functions
#
#  - - - - - - - - - License information: - - - - - - - - -
#
#  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
#
#  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
#  non-exclusive license to use, modify, copy and integrate this
#  file into your software for the sole purpose of interfacing
#  with Yoctopuce products.
#
#  You may reproduce and distribute copies of this file in
#  source or object form, as long as the sole purpose of this
#  code is to interface with Yoctopuce products. You must retain
#  this notice in the distributed source file.
#
#  You should refer to Yoctopuce General Terms and Conditions
#  for additional information regarding your rights and
#  obligations.
#
#  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
#  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
#  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
#  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
#  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
#  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
#  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
#  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
#  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
#  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
#  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
#  WARRANTY, OR OTHERWISE.
#
# *********************************************************************


__docformat__ = 'restructuredtext en'
from yoctopuce.yocto_api import *


#--- (generated code: YInputCaptureData class start)
#noinspection PyProtectedMember
class YInputCaptureData(object):
    """
    InputCaptureData objects represent raw data
    sampled by the analog/digital converter present in
    a Yoctopuce electrical sensor. When several inputs
    are samples simultaneously, their data are provided
    as distinct series.

    """
    #--- (end of generated code: YInputCaptureData class start)
    #--- (generated code: YInputCaptureData return codes)
    #--- (end of generated code: YInputCaptureData return codes)
    #--- (generated code: YInputCaptureData dlldef)
    #--- (end of generated code: YInputCaptureData dlldef)
    #--- (generated code: YInputCaptureData yapiwrapper)
    #--- (end of generated code: YInputCaptureData yapiwrapper)
    #--- (generated code: YInputCaptureData definitions)
    #--- (end of generated code: YInputCaptureData definitions)

    def __init__(self, yfun, sdata):
        #--- (generated code: YInputCaptureData attributes)
        self._fmt = 0
        self._var1size = 0
        self._var2size = 0
        self._var3size = 0
        self._nVars = 0
        self._recOfs = 0
        self._nRecs = 0
        self._samplesPerSec = 0
        self._trigType = 0
        self._trigVal = 0
        self._trigPos = 0
        self._trigUTC = 0
        self._var1unit = ''
        self._var2unit = ''
        self._var3unit = ''
        self._var1samples = []
        self._var2samples = []
        self._var3samples = []
        #--- (end of generated code: YInputCaptureData attributes)
        self._decodeSnapBin(sdata)

    def _throw(self, errType, errorMessage):
        if not YAPI.ExceptionsDisabled:
            raise YAPI.YAPI_Exception(errType, "YoctoApi error : " + errorMessage)

    #--- (generated code: YInputCaptureData implementation)
    def _decodeU16(self, sdata, ofs):
        # v
        v = YGetByte(sdata, ofs)
        v = v + 256 * YGetByte(sdata, ofs+1)
        return v

    def _decodeU32(self, sdata, ofs):
        # v
        v = self._decodeU16(sdata, ofs)
        v = v + 65536.0 * self._decodeU16(sdata, ofs+2)
        return v

    def _decodeVal(self, sdata, ofs, len):
        # v
        # b
        v = self._decodeU16(sdata, ofs)
        b = 65536.0
        ofs = ofs + 2
        len = len - 2
        while len > 0:
            v = v + b * YGetByte(sdata, ofs)
            b = b * 256
            ofs = ofs + 1
            len = len - 1
        if v > (b/2):
            # // negative number
            v = v - b
        return v

    def _decodeSnapBin(self, sdata):
        # buffSize
        # recOfs
        # ms
        # recSize
        # count
        # mult1
        # mult2
        # mult3
        # v

        buffSize = len(sdata)
        if not (buffSize >= 24):
            self._throw(YAPI.INVALID_ARGUMENT, "Invalid snapshot data (too short)")
            return YAPI.INVALID_ARGUMENT
        self._fmt = YGetByte(sdata, 0)
        self._var1size = YGetByte(sdata, 1) - 48
        self._var2size = YGetByte(sdata, 2) - 48
        self._var3size = YGetByte(sdata, 3) - 48
        if not (self._fmt == 83):
            self._throw(YAPI.INVALID_ARGUMENT, "Unsupported snapshot format")
            return YAPI.INVALID_ARGUMENT
        if not ((self._var1size >= 2) and (self._var1size <= 4)):
            self._throw(YAPI.INVALID_ARGUMENT, "Invalid sample size")
            return YAPI.INVALID_ARGUMENT
        if not ((self._var2size >= 0) and (self._var1size <= 4)):
            self._throw(YAPI.INVALID_ARGUMENT, "Invalid sample size")
            return YAPI.INVALID_ARGUMENT
        if not ((self._var3size >= 0) and (self._var1size <= 4)):
            self._throw(YAPI.INVALID_ARGUMENT, "Invalid sample size")
            return YAPI.INVALID_ARGUMENT
        if self._var2size == 0:
            self._nVars = 1
        else:
            if self._var3size == 0:
                self._nVars = 2
            else:
                self._nVars = 3
        recSize = self._var1size + self._var2size + self._var3size
        self._recOfs = self._decodeU16(sdata, 4)
        self._nRecs = self._decodeU16(sdata, 6)
        self._samplesPerSec = self._decodeU16(sdata, 8)
        self._trigType = self._decodeU16(sdata, 10)
        self._trigVal = self._decodeVal(sdata, 12, 4) / 1000
        self._trigPos = self._decodeU16(sdata, 16)
        ms = self._decodeU16(sdata, 18)
        self._trigUTC = self._decodeVal(sdata, 20, 4)
        self._trigUTC = self._trigUTC + (ms / 1000.0)
        recOfs = 24
        while YGetByte(sdata, recOfs) >= 32:
            self._var1unit = "" + self._var1unit + "" + str(chr(YGetByte(sdata, recOfs)))
            recOfs = recOfs + 1
        if self._var2size > 0:
            recOfs = recOfs + 1
            while YGetByte(sdata, recOfs) >= 32:
                self._var2unit = "" + self._var2unit + "" + str(chr(YGetByte(sdata, recOfs)))
                recOfs = recOfs + 1
        if self._var3size > 0:
            recOfs = recOfs + 1
            while YGetByte(sdata, recOfs) >= 32:
                self._var3unit = "" + self._var3unit + "" + str(chr(YGetByte(sdata, recOfs)))
                recOfs = recOfs + 1
        if ((recOfs) & (1)) == 1:
            # // align to next word
            recOfs = recOfs + 1
        mult1 = 1
        mult2 = 1
        mult3 = 1
        if recOfs < self._recOfs:
            # // load optional value multiplier
            mult1 = self._decodeU16(sdata, self._recOfs)
            recOfs = recOfs + 2
            if self._var2size > 0:
                mult2 = self._decodeU16(sdata, self._recOfs)
                recOfs = recOfs + 2
            if self._var3size > 0:
                mult3 = self._decodeU16(sdata, self._recOfs)
                recOfs = recOfs + 2

        recOfs = self._recOfs
        count = self._nRecs
        while (count > 0) and (recOfs + self._var1size <= buffSize):
            v = self._decodeVal(sdata, recOfs, self._var1size) / 1000.0
            self._var1samples.append(v*mult1)
            recOfs = recOfs + recSize

        if self._var2size > 0:
            recOfs = self._recOfs + self._var1size
            count = self._nRecs
            while (count > 0) and (recOfs + self._var2size <= buffSize):
                v = self._decodeVal(sdata, recOfs, self._var2size) / 1000.0
                self._var2samples.append(v*mult2)
                recOfs = recOfs + recSize
        if self._var3size > 0:
            recOfs = self._recOfs + self._var1size + self._var2size
            count = self._nRecs
            while (count > 0) and (recOfs + self._var3size <= buffSize):
                v = self._decodeVal(sdata, recOfs, self._var3size) / 1000.0
                self._var3samples.append(v*mult3)
                recOfs = recOfs + recSize
        return YAPI.SUCCESS

    def get_serieCount(self):
        """
        Returns the number of series available in the capture.

        @return an integer corresponding to the number of
                simultaneous data series available.
        """
        return self._nVars

    def get_recordCount(self):
        """
        Returns the number of records captured (in a serie).
        In the exceptional case where it was not possible
        to transfer all data in time, the number of records
        actually present in the series might be lower than
        the number of records captured

        @return an integer corresponding to the number of
                records expected in each serie.
        """
        return self._nRecs

    def get_samplingRate(self):
        """
        Returns the effective sampling rate of the device.

        @return an integer corresponding to the number of
                samples taken each second.
        """
        return self._samplesPerSec

    def get_captureType(self):
        """
        Returns the type of automatic conditional capture
        that triggered the capture of this data sequence.

        @return the type of conditional capture.
        """
        return int(self._trigType)

    def get_triggerValue(self):
        """
        Returns the threshold value that triggered
        this automatic conditional capture, if it was
        not an instant captured triggered manually.

        @return the conditional threshold value
                at the time of capture.
        """
        return self._trigVal

    def get_triggerPosition(self):
        """
        Returns the index in the series of the sample
        corresponding to the exact time when the capture
        was triggered. In case of trigger based on average
        or RMS value, the trigger index corresponds to
        the end of the averaging period.

        @return an integer corresponding to a position
                in the data serie.
        """
        return self._trigPos

    def get_triggerRealTimeUTC(self):
        """
        Returns the absolute time when the capture was
        triggered, as a Unix timestamp. Milliseconds are
        included in this timestamp (floating-point number).

        @return a floating-point number corresponding to
                the number of seconds between the Jan 1,
                1970 and the moment where the capture
                was triggered.
        """
        return self._trigUTC

    def get_serie1Unit(self):
        """
        Returns the unit of measurement for data points in
        the first serie.

        @return a string containing to a physical unit of
                measurement.
        """
        return self._var1unit

    def get_serie2Unit(self):
        """
        Returns the unit of measurement for data points in
        the second serie.

        @return a string containing to a physical unit of
                measurement.
        """
        if not (self._nVars >= 2):
            self._throw(YAPI.INVALID_ARGUMENT, "There is no serie 2 in this capture data")
            return ""
        return self._var2unit

    def get_serie3Unit(self):
        """
        Returns the unit of measurement for data points in
        the third serie.

        @return a string containing to a physical unit of
                measurement.
        """
        if not (self._nVars >= 3):
            self._throw(YAPI.INVALID_ARGUMENT, "There is no serie 3 in this capture data")
            return ""
        return self._var3unit

    def get_serie1Values(self):
        """
        Returns the sampled data corresponding to the first serie.
        The corresponding physical unit can be obtained
        using the method get_serie1Unit().

        @return a list of real numbers corresponding to all
                samples received for serie 1.

        On failure, throws an exception or returns an empty array.
        """
        return self._var1samples

    def get_serie2Values(self):
        """
        Returns the sampled data corresponding to the second serie.
        The corresponding physical unit can be obtained
        using the method get_serie2Unit().

        @return a list of real numbers corresponding to all
                samples received for serie 2.

        On failure, throws an exception or returns an empty array.
        """
        if not (self._nVars >= 2):
            self._throw(YAPI.INVALID_ARGUMENT, "There is no serie 2 in this capture data")
            return self._var2samples
        return self._var2samples

    def get_serie3Values(self):
        """
        Returns the sampled data corresponding to the third serie.
        The corresponding physical unit can be obtained
        using the method get_serie3Unit().

        @return a list of real numbers corresponding to all
                samples received for serie 3.

        On failure, throws an exception or returns an empty array.
        """
        if not (self._nVars >= 3):
            self._throw(YAPI.INVALID_ARGUMENT, "There is no serie 3 in this capture data")
            return self._var3samples
        return self._var3samples

#--- (end of generated code: YInputCaptureData implementation)

#--- (generated code: YInputCaptureData functions)
#--- (end of generated code: YInputCaptureData functions)


#--- (generated code: YInputCapture class start)
#noinspection PyProtectedMember
class YInputCapture(YFunction):
    """
    The YInputCapture class allows you to access data samples
    measured by a Yoctopuce electrical sensor. The data capture can be
    triggered manually, or be configured to detect specific events.

    """
    #--- (end of generated code: YInputCapture class start)
    #--- (generated code: YInputCapture return codes)
    #--- (end of generated code: YInputCapture return codes)
    #--- (generated code: YInputCapture dlldef)
    #--- (end of generated code: YInputCapture dlldef)
    #--- (generated code: YInputCapture yapiwrapper)
    #--- (end of generated code: YInputCapture yapiwrapper)
    #--- (generated code: YInputCapture definitions)
    LASTCAPTURETIME_INVALID = YAPI.INVALID_LONG
    NSAMPLES_INVALID = YAPI.INVALID_UINT
    SAMPLINGRATE_INVALID = YAPI.INVALID_UINT
    CONDVALUE_INVALID = YAPI.INVALID_DOUBLE
    CONDALIGN_INVALID = YAPI.INVALID_UINT
    CONDVALUEATSTARTUP_INVALID = YAPI.INVALID_DOUBLE
    CAPTURETYPE_NONE = 0
    CAPTURETYPE_TIMED = 1
    CAPTURETYPE_V_MAX = 2
    CAPTURETYPE_V_MIN = 3
    CAPTURETYPE_I_MAX = 4
    CAPTURETYPE_I_MIN = 5
    CAPTURETYPE_P_MAX = 6
    CAPTURETYPE_P_MIN = 7
    CAPTURETYPE_V_AVG_MAX = 8
    CAPTURETYPE_V_AVG_MIN = 9
    CAPTURETYPE_V_RMS_MAX = 10
    CAPTURETYPE_V_RMS_MIN = 11
    CAPTURETYPE_I_AVG_MAX = 12
    CAPTURETYPE_I_AVG_MIN = 13
    CAPTURETYPE_I_RMS_MAX = 14
    CAPTURETYPE_I_RMS_MIN = 15
    CAPTURETYPE_P_AVG_MAX = 16
    CAPTURETYPE_P_AVG_MIN = 17
    CAPTURETYPE_PF_MIN = 18
    CAPTURETYPE_DPF_MIN = 19
    CAPTURETYPE_INVALID = -1
    CAPTURETYPEATSTARTUP_NONE = 0
    CAPTURETYPEATSTARTUP_TIMED = 1
    CAPTURETYPEATSTARTUP_V_MAX = 2
    CAPTURETYPEATSTARTUP_V_MIN = 3
    CAPTURETYPEATSTARTUP_I_MAX = 4
    CAPTURETYPEATSTARTUP_I_MIN = 5
    CAPTURETYPEATSTARTUP_P_MAX = 6
    CAPTURETYPEATSTARTUP_P_MIN = 7
    CAPTURETYPEATSTARTUP_V_AVG_MAX = 8
    CAPTURETYPEATSTARTUP_V_AVG_MIN = 9
    CAPTURETYPEATSTARTUP_V_RMS_MAX = 10
    CAPTURETYPEATSTARTUP_V_RMS_MIN = 11
    CAPTURETYPEATSTARTUP_I_AVG_MAX = 12
    CAPTURETYPEATSTARTUP_I_AVG_MIN = 13
    CAPTURETYPEATSTARTUP_I_RMS_MAX = 14
    CAPTURETYPEATSTARTUP_I_RMS_MIN = 15
    CAPTURETYPEATSTARTUP_P_AVG_MAX = 16
    CAPTURETYPEATSTARTUP_P_AVG_MIN = 17
    CAPTURETYPEATSTARTUP_PF_MIN = 18
    CAPTURETYPEATSTARTUP_DPF_MIN = 19
    CAPTURETYPEATSTARTUP_INVALID = -1
    #--- (end of generated code: YInputCapture definitions)

    def __init__(self, func):
        super(YInputCapture, self).__init__(func)
        self._className = 'InputCapture'
        #--- (generated code: YInputCapture attributes)
        self._callback = None
        self._lastCaptureTime = YInputCapture.LASTCAPTURETIME_INVALID
        self._nSamples = YInputCapture.NSAMPLES_INVALID
        self._samplingRate = YInputCapture.SAMPLINGRATE_INVALID
        self._captureType = YInputCapture.CAPTURETYPE_INVALID
        self._condValue = YInputCapture.CONDVALUE_INVALID
        self._condAlign = YInputCapture.CONDALIGN_INVALID
        self._captureTypeAtStartup = YInputCapture.CAPTURETYPEATSTARTUP_INVALID
        self._condValueAtStartup = YInputCapture.CONDVALUEATSTARTUP_INVALID
        #--- (end of generated code: YInputCapture attributes)

    #--- (generated code: YInputCapture implementation)
    def _parseAttr(self, json_val):
        if json_val.has("lastCaptureTime"):
            self._lastCaptureTime = json_val.getLong("lastCaptureTime")
        if json_val.has("nSamples"):
            self._nSamples = json_val.getInt("nSamples")
        if json_val.has("samplingRate"):
            self._samplingRate = json_val.getInt("samplingRate")
        if json_val.has("captureType"):
            self._captureType = json_val.getInt("captureType")
        if json_val.has("condValue"):
            self._condValue = round(json_val.getDouble("condValue") / 65.536) / 1000.0
        if json_val.has("condAlign"):
            self._condAlign = json_val.getInt("condAlign")
        if json_val.has("captureTypeAtStartup"):
            self._captureTypeAtStartup = json_val.getInt("captureTypeAtStartup")
        if json_val.has("condValueAtStartup"):
            self._condValueAtStartup = round(json_val.getDouble("condValueAtStartup") / 65.536) / 1000.0
        super(YInputCapture, self)._parseAttr(json_val)

    def get_lastCaptureTime(self):
        """
        Returns the number of elapsed milliseconds between the module power on
        and the last capture (time of trigger), or zero if no capture has been done.

        @return an integer corresponding to the number of elapsed milliseconds between the module power on
                and the last capture (time of trigger), or zero if no capture has been done

        On failure, throws an exception or returns YInputCapture.LASTCAPTURETIME_INVALID.
        """
        # res
        if self._cacheExpiration <= YAPI.GetTickCount():
            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                return YInputCapture.LASTCAPTURETIME_INVALID
        res = self._lastCaptureTime
        return res

    def get_nSamples(self):
        """
        Returns the number of samples that will be captured.

        @return an integer corresponding to the number of samples that will be captured

        On failure, throws an exception or returns YInputCapture.NSAMPLES_INVALID.
        """
        # res
        if self._cacheExpiration <= YAPI.GetTickCount():
            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                return YInputCapture.NSAMPLES_INVALID
        res = self._nSamples
        return res

    def set_nSamples(self, newval):
        """
        Changes the type of automatic conditional capture.
        The maximum number of samples depends on the device memory.

        If you want the change to be kept after a device reboot,
        make sure  to call the matching module saveToFlash().

        @param newval : an integer corresponding to the type of automatic conditional capture

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = str(newval)
        return self._setAttr("nSamples", rest_val)

    def get_samplingRate(self):
        """
        Returns the sampling frequency, in Hz.

        @return an integer corresponding to the sampling frequency, in Hz

        On failure, throws an exception or returns YInputCapture.SAMPLINGRATE_INVALID.
        """
        # res
        if self._cacheExpiration <= YAPI.GetTickCount():
            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                return YInputCapture.SAMPLINGRATE_INVALID
        res = self._samplingRate
        return res

    def get_captureType(self):
        """
        Returns the type of automatic conditional capture.

        @return a value among YInputCapture.CAPTURETYPE_NONE, YInputCapture.CAPTURETYPE_TIMED,
        YInputCapture.CAPTURETYPE_V_MAX, YInputCapture.CAPTURETYPE_V_MIN, YInputCapture.CAPTURETYPE_I_MAX,
        YInputCapture.CAPTURETYPE_I_MIN, YInputCapture.CAPTURETYPE_P_MAX, YInputCapture.CAPTURETYPE_P_MIN,
        YInputCapture.CAPTURETYPE_V_AVG_MAX, YInputCapture.CAPTURETYPE_V_AVG_MIN,
        YInputCapture.CAPTURETYPE_V_RMS_MAX, YInputCapture.CAPTURETYPE_V_RMS_MIN,
        YInputCapture.CAPTURETYPE_I_AVG_MAX, YInputCapture.CAPTURETYPE_I_AVG_MIN,
        YInputCapture.CAPTURETYPE_I_RMS_MAX, YInputCapture.CAPTURETYPE_I_RMS_MIN,
        YInputCapture.CAPTURETYPE_P_AVG_MAX, YInputCapture.CAPTURETYPE_P_AVG_MIN,
        YInputCapture.CAPTURETYPE_PF_MIN and YInputCapture.CAPTURETYPE_DPF_MIN corresponding to the type of
        automatic conditional capture

        On failure, throws an exception or returns YInputCapture.CAPTURETYPE_INVALID.
        """
        # res
        if self._cacheExpiration <= YAPI.GetTickCount():
            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                return YInputCapture.CAPTURETYPE_INVALID
        res = self._captureType
        return res

    def set_captureType(self, newval):
        """
        Changes the type of automatic conditional capture.

        @param newval : a value among YInputCapture.CAPTURETYPE_NONE, YInputCapture.CAPTURETYPE_TIMED,
        YInputCapture.CAPTURETYPE_V_MAX, YInputCapture.CAPTURETYPE_V_MIN, YInputCapture.CAPTURETYPE_I_MAX,
        YInputCapture.CAPTURETYPE_I_MIN, YInputCapture.CAPTURETYPE_P_MAX, YInputCapture.CAPTURETYPE_P_MIN,
        YInputCapture.CAPTURETYPE_V_AVG_MAX, YInputCapture.CAPTURETYPE_V_AVG_MIN,
        YInputCapture.CAPTURETYPE_V_RMS_MAX, YInputCapture.CAPTURETYPE_V_RMS_MIN,
        YInputCapture.CAPTURETYPE_I_AVG_MAX, YInputCapture.CAPTURETYPE_I_AVG_MIN,
        YInputCapture.CAPTURETYPE_I_RMS_MAX, YInputCapture.CAPTURETYPE_I_RMS_MIN,
        YInputCapture.CAPTURETYPE_P_AVG_MAX, YInputCapture.CAPTURETYPE_P_AVG_MIN,
        YInputCapture.CAPTURETYPE_PF_MIN and YInputCapture.CAPTURETYPE_DPF_MIN corresponding to the type of
        automatic conditional capture

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = str(newval)
        return self._setAttr("captureType", rest_val)

    def set_condValue(self, newval):
        """
        Changes current threshold value for automatic conditional capture.

        @param newval : a floating point number corresponding to current threshold value for automatic
        conditional capture

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = str(int(round(newval * 65536.0, 1)))
        return self._setAttr("condValue", rest_val)

    def get_condValue(self):
        """
        Returns current threshold value for automatic conditional capture.

        @return a floating point number corresponding to current threshold value for automatic conditional capture

        On failure, throws an exception or returns YInputCapture.CONDVALUE_INVALID.
        """
        # res
        if self._cacheExpiration <= YAPI.GetTickCount():
            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                return YInputCapture.CONDVALUE_INVALID
        res = self._condValue
        return res

    def get_condAlign(self):
        """
        Returns the relative position of the trigger event within the capture window.
        When the value is 50%, the capture is centered on the event.

        @return an integer corresponding to the relative position of the trigger event within the capture window

        On failure, throws an exception or returns YInputCapture.CONDALIGN_INVALID.
        """
        # res
        if self._cacheExpiration <= YAPI.GetTickCount():
            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                return YInputCapture.CONDALIGN_INVALID
        res = self._condAlign
        return res

    def set_condAlign(self, newval):
        """
        Changes the relative position of the trigger event within the capture window.
        The new value must be between 10% (on the left) and 90% (on the right).
        When the value is 50%, the capture is centered on the event.

        If you want the change to be kept after a device reboot,
        make sure  to call the matching module saveToFlash().

        @param newval : an integer corresponding to the relative position of the trigger event within the capture window

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = str(newval)
        return self._setAttr("condAlign", rest_val)

    def get_captureTypeAtStartup(self):
        """
        Returns the type of automatic conditional capture
        applied at device power on.

        @return a value among YInputCapture.CAPTURETYPEATSTARTUP_NONE,
        YInputCapture.CAPTURETYPEATSTARTUP_TIMED, YInputCapture.CAPTURETYPEATSTARTUP_V_MAX,
        YInputCapture.CAPTURETYPEATSTARTUP_V_MIN, YInputCapture.CAPTURETYPEATSTARTUP_I_MAX,
        YInputCapture.CAPTURETYPEATSTARTUP_I_MIN, YInputCapture.CAPTURETYPEATSTARTUP_P_MAX,
        YInputCapture.CAPTURETYPEATSTARTUP_P_MIN, YInputCapture.CAPTURETYPEATSTARTUP_V_AVG_MAX,
        YInputCapture.CAPTURETYPEATSTARTUP_V_AVG_MIN, YInputCapture.CAPTURETYPEATSTARTUP_V_RMS_MAX,
        YInputCapture.CAPTURETYPEATSTARTUP_V_RMS_MIN, YInputCapture.CAPTURETYPEATSTARTUP_I_AVG_MAX,
        YInputCapture.CAPTURETYPEATSTARTUP_I_AVG_MIN, YInputCapture.CAPTURETYPEATSTARTUP_I_RMS_MAX,
        YInputCapture.CAPTURETYPEATSTARTUP_I_RMS_MIN, YInputCapture.CAPTURETYPEATSTARTUP_P_AVG_MAX,
        YInputCapture.CAPTURETYPEATSTARTUP_P_AVG_MIN, YInputCapture.CAPTURETYPEATSTARTUP_PF_MIN and
        YInputCapture.CAPTURETYPEATSTARTUP_DPF_MIN corresponding to the type of automatic conditional capture
                applied at device power on

        On failure, throws an exception or returns YInputCapture.CAPTURETYPEATSTARTUP_INVALID.
        """
        # res
        if self._cacheExpiration <= YAPI.GetTickCount():
            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                return YInputCapture.CAPTURETYPEATSTARTUP_INVALID
        res = self._captureTypeAtStartup
        return res

    def set_captureTypeAtStartup(self, newval):
        """
        Changes the type of automatic conditional capture
        applied at device power on.

        If you want the change to be kept after a device reboot,
        make sure  to call the matching module saveToFlash().

        @param newval : a value among YInputCapture.CAPTURETYPEATSTARTUP_NONE,
        YInputCapture.CAPTURETYPEATSTARTUP_TIMED, YInputCapture.CAPTURETYPEATSTARTUP_V_MAX,
        YInputCapture.CAPTURETYPEATSTARTUP_V_MIN, YInputCapture.CAPTURETYPEATSTARTUP_I_MAX,
        YInputCapture.CAPTURETYPEATSTARTUP_I_MIN, YInputCapture.CAPTURETYPEATSTARTUP_P_MAX,
        YInputCapture.CAPTURETYPEATSTARTUP_P_MIN, YInputCapture.CAPTURETYPEATSTARTUP_V_AVG_MAX,
        YInputCapture.CAPTURETYPEATSTARTUP_V_AVG_MIN, YInputCapture.CAPTURETYPEATSTARTUP_V_RMS_MAX,
        YInputCapture.CAPTURETYPEATSTARTUP_V_RMS_MIN, YInputCapture.CAPTURETYPEATSTARTUP_I_AVG_MAX,
        YInputCapture.CAPTURETYPEATSTARTUP_I_AVG_MIN, YInputCapture.CAPTURETYPEATSTARTUP_I_RMS_MAX,
        YInputCapture.CAPTURETYPEATSTARTUP_I_RMS_MIN, YInputCapture.CAPTURETYPEATSTARTUP_P_AVG_MAX,
        YInputCapture.CAPTURETYPEATSTARTUP_P_AVG_MIN, YInputCapture.CAPTURETYPEATSTARTUP_PF_MIN and
        YInputCapture.CAPTURETYPEATSTARTUP_DPF_MIN corresponding to the type of automatic conditional capture
                applied at device power on

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = str(newval)
        return self._setAttr("captureTypeAtStartup", rest_val)

    def set_condValueAtStartup(self, newval):
        """
        Changes current threshold value for automatic conditional
        capture applied at device power on.

        If you want the change to be kept after a device reboot,
        make sure  to call the matching module saveToFlash().

        @param newval : a floating point number corresponding to current threshold value for automatic conditional
                capture applied at device power on

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = str(int(round(newval * 65536.0, 1)))
        return self._setAttr("condValueAtStartup", rest_val)

    def get_condValueAtStartup(self):
        """
        Returns the threshold value for automatic conditional
        capture applied at device power on.

        @return a floating point number corresponding to the threshold value for automatic conditional
                capture applied at device power on

        On failure, throws an exception or returns YInputCapture.CONDVALUEATSTARTUP_INVALID.
        """
        # res
        if self._cacheExpiration <= YAPI.GetTickCount():
            if self.load(YAPI._yapiContext.GetCacheValidity()) != YAPI.SUCCESS:
                return YInputCapture.CONDVALUEATSTARTUP_INVALID
        res = self._condValueAtStartup
        return res

    @staticmethod
    def FindInputCapture(func):
        """
        Retrieves an instant snapshot trigger for a given identifier.
        The identifier can be specified using several formats:
        <ul>
        <li>FunctionLogicalName</li>
        <li>ModuleSerialNumber.FunctionIdentifier</li>
        <li>ModuleSerialNumber.FunctionLogicalName</li>
        <li>ModuleLogicalName.FunctionIdentifier</li>
        <li>ModuleLogicalName.FunctionLogicalName</li>
        </ul>

        This function does not require that the instant snapshot trigger is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YInputCapture.isOnline() to test if the instant snapshot trigger is
        indeed online at a given time. In case of ambiguity when looking for
        an instant snapshot trigger by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        If a call to this object's is_online() method returns FALSE although
        you are certain that the matching device is plugged, make sure that you did
        call registerHub() at application initialization time.

        @param func : a string that uniquely characterizes the instant snapshot trigger, for instance
                MyDevice.inputCapture.

        @return a YInputCapture object allowing you to drive the instant snapshot trigger.
        """
        # obj
        obj = YFunction._FindFromCache("InputCapture", func)
        if obj is None:
            obj = YInputCapture(func)
            YFunction._AddToCache("InputCapture", func, obj)
        return obj

    def get_lastCapture(self):
        """
        Returns all details about the last automatic input capture.

        @return an YInputCaptureData object including
                data series and all related meta-information.
                On failure, throws an exception or returns an capture object.
        """
        # snapData

        snapData = self._download("snap.bin")
        return YInputCaptureData(self, snapData)

    def get_immediateCapture(self, msDuration):
        """
        Returns a new immediate capture of the device inputs.

        @param msDuration : duration of the capture window,
                in milliseconds (eg. between 20 and 1000).

        @return an YInputCaptureData object including
                data series for the specified duration.
                On failure, throws an exception or returns an capture object.
        """
        # snapUrl
        # snapData
        # snapStart
        if msDuration < 1:
            msDuration = 20
        if msDuration > 1000:
            msDuration = 1000
        snapStart = int((-msDuration) / (2))
        snapUrl = "snap.bin?t=" + str(int(snapStart)) + "&d=" + str(int(msDuration))

        snapData = self._download(snapUrl)
        return YInputCaptureData(self, snapData)

    def nextInputCapture(self):
        """
        comment from .yc definition
        """
        hwidRef = YRefParam()
        if YAPI.YISERR(self._nextFunction(hwidRef)):
            return None
        if hwidRef.value == "":
            return None
        return YInputCapture.FindInputCapture(hwidRef.value)

#--- (end of generated code: YInputCapture implementation)

#--- (generated code: YInputCapture functions)

    @staticmethod
    def FirstInputCapture():
        """
        comment from .yc definition
        """
        devRef = YRefParam()
        neededsizeRef = YRefParam()
        serialRef = YRefParam()
        funcIdRef = YRefParam()
        funcNameRef = YRefParam()
        funcValRef = YRefParam()
        errmsgRef = YRefParam()
        size = YAPI.C_INTSIZE
        #noinspection PyTypeChecker,PyCallingNonCallable
        p = (ctypes.c_int * 1)()
        err = YAPI.apiGetFunctionsByClass("InputCapture", 0, p, size, neededsizeRef, errmsgRef)

        if YAPI.YISERR(err) or not neededsizeRef.value:
            return None

        if YAPI.YISERR(
                YAPI.yapiGetFunctionInfo(p[0], devRef, serialRef, funcIdRef, funcNameRef, funcValRef, errmsgRef)):
            return None

        return YInputCapture.FindInputCapture(serialRef.value + "." + funcIdRef.value)

#--- (end of generated code: YInputCapture functions)
